@@ -1,21 +1,16 @@
 <?php
 /*
-** Zabbix
-** Copyright (C) 2001-2018 Zabbix SIA
+** Copyright (C) 2001-2025 Zabbix SIA
 **
-** This program is free software; you can redistribute it and/or modify
-** it under the terms of the GNU General Public License as published by
-** the Free Software Foundation; either version 2 of the License, or
-** (at your option) any later version.
+** This program is free software: you can redistribute it and/or modify it under the terms of
+** the GNU Affero General Public License as published by the Free Software Foundation, version 3.
 **
-** This program is distributed in the hope that it will be useful,
-** but WITHOUT ANY WARRANTY; without even the implied warranty of
-** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-** GNU General Public License for more details.
+** This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
+** without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+** See the GNU Affero General Public License for more details.
 **
-** You should have received a copy of the GNU General Public License
-** along with this program; if not, write to the Free Software
-** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+** You should have received a copy of the GNU Affero General Public License along with this program.
+** If not, see <https://www.gnu.org/licenses/>.
 **/
 
 
@@ -24,6 +19,13 @@
  */
 class CDRule extends CApiService {
 
+	public const ACCESS_RULES = [
+		'get' => ['min_user_type' => USER_TYPE_ZABBIX_USER],
+		'create' => ['min_user_type' => USER_TYPE_ZABBIX_ADMIN],
+		'update' => ['min_user_type' => USER_TYPE_ZABBIX_ADMIN],
+		'delete' => ['min_user_type' => USER_TYPE_ZABBIX_ADMIN]
+	];
+
 	protected $tableName = 'drules';
 	protected $tableAlias = 'dr';
 	protected $sortColumns = ['druleid', 'name'];
@@ -139,7 +141,8 @@ class CDRule extends CApiService {
 
 		$sqlParts = $this->applyQueryOutputOptions($this->tableName(), $this->tableAlias(), $options, $sqlParts);
 		$sqlParts = $this->applyQuerySortOptions($this->tableName(), $this->tableAlias(), $options, $sqlParts);
-		$dbRes = DBselect($this->createSelectQueryFromParts($sqlParts), $sqlParts['limit']);
+		$dbRes = DBselect(self::createSelectQueryFromParts($sqlParts), $sqlParts['limit']);
+
 		while ($drule = DBfetch($dbRes)) {
 			if ($options['countOutput']) {
 				if ($options['groupCount']) {
@@ -160,14 +163,13 @@ class CDRule extends CApiService {
 
 		if ($result) {
 			$result = $this->addRelatedObjects($options, $result);
+
+			if (!$options['preservekeys']) {
+				$result = array_values($result);
+			}
 		}
 
-// removing keys (hash -> array)
-		if (!$options['preservekeys']) {
-			$result = zbx_cleanHashes($result);
-		}
-
-	return $result;
+		return $result;
 	}
 
 	/**
@@ -187,11 +189,16 @@ class CDRule extends CApiService {
 			self::exception(ZBX_API_ERROR_PARAMETERS, _('Empty input parameter.'));
 		}
 
-		$proxy_hostids = [];
+		$proxyids = [];
 
 		$ip_range_parser = new CIPRangeParser(['v6' => ZBX_HAVE_IPV6, 'dns' => false, 'max_ipv4_cidr' => 30]);
+		$allowed_fields = array_flip(['proxyid', 'name', 'iprange', 'delay', 'status', 'concurrency_max', 'dchecks']);
 
 		foreach ($drules as $drule) {
+			if (array_diff_key($drule, $allowed_fields)) {
+				self::exception(ZBX_API_ERROR_PARAMETERS, _('Incorrect arguments passed to function.'));
+			}
+
 			if (!array_key_exists('name', $drule)) {
 				self::exception(ZBX_API_ERROR_PARAMETERS, _s('Field "%1$s" is mandatory.', 'name'));
 			}
@@ -238,15 +245,23 @@ class CDRule extends CApiService {
 				);
 			}
 
-			if (array_key_exists('proxy_hostid', $drule)) {
-				if (!zbx_is_int($drule['proxy_hostid'])) {
+			if (array_key_exists('concurrency_max', $drule)
+					&& ($drule['concurrency_max'] < ZBX_DISCOVERY_CHECKS_UNLIMITED
+						|| $drule['concurrency_max'] > ZBX_DISCOVERY_CHECKS_MAX)) {
+				self::exception(ZBX_API_ERROR_PARAMETERS,
+					_s('Incorrect value "%1$s" for "%2$s" field.', $drule['concurrency_max'], 'concurrency_max')
+				);
+			}
+
+			if (array_key_exists('proxyid', $drule)) {
+				if (!zbx_is_int($drule['proxyid'])) {
 					self::exception(ZBX_API_ERROR_PARAMETERS,
-						_s('Incorrect value "%1$s" for "%2$s" field.', $drule['proxy_hostid'], 'proxy_hostid')
+						_s('Incorrect value "%1$s" for "%2$s" field.', $drule['proxyid'], 'proxyid')
 					);
 				}
 
-				if ($drule['proxy_hostid'] > 0) {
-					$proxy_hostids[] = $drule['proxy_hostid'];
+				if ($drule['proxyid'] > 0) {
+					$proxyids[] = $drule['proxyid'];
 				}
 			}
 
@@ -260,6 +275,7 @@ class CDRule extends CApiService {
 
 		// Check drule name duplicates in input data.
 		$duplicate = CArrayHelper::findDuplicate($drules, 'name');
+
 		if ($duplicate) {
 			self::exception(ZBX_API_ERROR_PARAMETERS,
 				_s('Discovery rule "%1$s" already exists.', $duplicate['name'])
@@ -280,16 +296,17 @@ class CDRule extends CApiService {
 		}
 
 		// Check proxy IDs.
-		if ($proxy_hostids) {
-			$db_proxies = API::proxy()->get([
+		if ($proxyids) {
+			$db_proxies = API::Proxy()->get([
 				'output' => ['proxyid'],
-				'proxyids' => $proxy_hostids,
+				'proxyids' => $proxyids,
 				'preservekeys' => true
 			]);
-			foreach ($proxy_hostids as $proxy_hostid) {
-				if (!array_key_exists($proxy_hostid, $db_proxies)) {
+
+			foreach ($proxyids as $proxyid) {
+				if (!array_key_exists($proxyid, $db_proxies)) {
 					self::exception(ZBX_API_ERROR_PARAMETERS,
-						_s('Incorrect value "%1$s" for "%2$s" field.', $proxy_hostid, 'proxy_hostid')
+						_s('Incorrect value "%1$s" for "%2$s" field.', $proxyid, 'proxyid')
 					);
 				}
 			}
@@ -327,11 +344,18 @@ class CDRule extends CApiService {
 		]);
 
 		$drule_names_changed = [];
-		$proxy_hostids = [];
+		$proxyids = [];
 
 		$ip_range_parser = new CIPRangeParser(['v6' => ZBX_HAVE_IPV6, 'dns' => false, 'max_ipv4_cidr' => 30]);
+		$allowed_fields = array_flip([
+			'druleid', 'proxyid', 'name', 'iprange', 'delay', 'status', 'concurrency_max', 'dchecks'
+		]);
 
 		foreach ($drules as $drule) {
+			if (array_diff_key($drule, $allowed_fields)) {
+				self::exception(ZBX_API_ERROR_PARAMETERS, _('Incorrect arguments passed to function.'));
+			}
+
 			if (!array_key_exists($drule['druleid'], $db_drules)) {
 				self::exception(ZBX_API_ERROR_PARAMETERS, _('No permissions to referred object or it does not exist!'));
 			}
@@ -387,15 +411,23 @@ class CDRule extends CApiService {
 				);
 			}
 
-			if (array_key_exists('proxy_hostid', $drule)) {
-				if (!zbx_is_int($drule['proxy_hostid'])) {
+			if (array_key_exists('concurrency_max', $drule)
+					&& ($drule['concurrency_max'] < ZBX_DISCOVERY_CHECKS_UNLIMITED
+						|| $drule['concurrency_max'] > ZBX_DISCOVERY_CHECKS_MAX)) {
+				self::exception(ZBX_API_ERROR_PARAMETERS,
+					_s('Incorrect value "%1$s" for "%2$s" field.', $drule['concurrency_max'], 'concurrency_max')
+				);
+			}
+
+			if (array_key_exists('proxyid', $drule)) {
+				if (!zbx_is_int($drule['proxyid'])) {
 					self::exception(ZBX_API_ERROR_PARAMETERS,
-						_s('Incorrect value "%1$s" for "%2$s" field.', $drule['proxy_hostid'], 'proxy_hostid')
+						_s('Incorrect value "%1$s" for "%2$s" field.', $drule['proxyid'], 'proxyid')
 					);
 				}
 
-				if ($drule['proxy_hostid'] > 0) {
-					$proxy_hostids[] = $drule['proxy_hostid'];
+				if ($drule['proxyid'] > 0) {
+					$proxyids[] = $drule['proxyid'];
 				}
 			}
 
@@ -412,6 +444,7 @@ class CDRule extends CApiService {
 		if ($drule_names_changed) {
 			// Check drule name duplicates in input data.
 			$duplicate = CArrayHelper::findDuplicate($drule_names_changed, 'name');
+
 			if ($duplicate) {
 				self::exception(ZBX_API_ERROR_PARAMETERS,
 					_s('Discovery rule "%1$s" already exists.', $duplicate['name'])
@@ -433,16 +466,17 @@ class CDRule extends CApiService {
 		}
 
 		// Check proxy IDs.
-		if ($proxy_hostids) {
-			$db_proxies = API::proxy()->get([
+		if ($proxyids) {
+			$db_proxies = API::Proxy()->get([
 				'output' => ['proxyid'],
-				'proxyids' => $proxy_hostids,
+				'proxyids' => $proxyids,
 				'preservekeys' => true
 			]);
-			foreach ($proxy_hostids as $proxy_hostid) {
-				if (!array_key_exists($proxy_hostid, $db_proxies)) {
+
+			foreach ($proxyids as $proxyid) {
+				if (!array_key_exists($proxyid, $db_proxies)) {
 					self::exception(ZBX_API_ERROR_PARAMETERS,
-						_s('Incorrect value "%1$s" for "%2$s" field.', $proxy_hostid, 'proxy_hostid')
+						_s('Incorrect value "%1$s" for "%2$s" field.', $proxyid, 'proxyid')
 					);
 				}
 			}
@@ -457,6 +491,10 @@ class CDRule extends CApiService {
 	protected function validateDChecks(array $dchecks) {
 		$uniq = 0;
 		$item_key_parser = new CItemKey();
+		$source_values = [
+			'name_source' => [ZBX_DISCOVERY_UNSPEC, ZBX_DISCOVERY_DNS, ZBX_DISCOVERY_IP, ZBX_DISCOVERY_VALUE],
+			'host_source' => [ZBX_DISCOVERY_DNS, ZBX_DISCOVERY_IP, ZBX_DISCOVERY_VALUE]
+		];
 
 		if (!is_array($dchecks)) {
 			self::exception(ZBX_API_ERROR_PARAMETERS,
@@ -485,6 +523,30 @@ class CDRule extends CApiService {
 				self::exception(ZBX_API_ERROR_PARAMETERS, _s('Incorrect port range.'));
 			}
 
+			foreach ($source_values as $field => $values) {
+				if (!array_key_exists($field, $dcheck)) {
+					continue;
+				}
+
+				if (!in_array($dcheck['type'], [SVC_AGENT, SVC_SNMPv1, SVC_SNMPv2c, SVC_SNMPv3])
+						&& $dcheck[$field] == ZBX_DISCOVERY_VALUE) {
+					self::exception(ZBX_API_ERROR_PARAMETERS,
+						_s('Incorrect value for field "%1$s": %2$s.', $field, $dcheck[$field])
+					);
+				}
+
+				if (!in_array($dcheck[$field], $values)) {
+					self::exception(ZBX_API_ERROR_PARAMETERS,
+						_s('Incorrect value for field "%1$s": %2$s.', $field, $dcheck[$field])
+					);
+				}
+
+				// Only one check can be equal ZBX_DISCOVERY_VALUE for 'host_source' and 'name_source' fields.
+				if ($dcheck[$field] == ZBX_DISCOVERY_VALUE) {
+					array_pop($source_values[$field]);
+				}
+			}
+
 			$dcheck_types = [SVC_SSH, SVC_LDAP, SVC_SMTP, SVC_FTP, SVC_HTTP, SVC_POP, SVC_NNTP, SVC_IMAP, SVC_TCP,
 				SVC_AGENT, SVC_SNMPv1, SVC_SNMPv2c, SVC_ICMPPING, SVC_SNMPv3, SVC_HTTPS, SVC_TELNET
 			];
@@ -497,6 +559,7 @@ class CDRule extends CApiService {
 					_s('Incorrect value "%1$s" for "%2$s" field.', $dcheck['type'], 'type')
 				);
 			}
+
 			switch ($dcheck['type']) {
 				case SVC_AGENT:
 					if (!array_key_exists('key_', $dcheck)) {
@@ -514,10 +577,12 @@ class CDRule extends CApiService {
 					}
 
 					$length = mb_strlen($dcheck['key_']);
-					if ($length > 255) {
+					$max_length = DB::getFieldLength('dchecks', 'key_');
+
+					if ($length > $max_length) {
 						self::exception(ZBX_API_ERROR_PARAMETERS,
 							_s('Incorrect value for field "%1$s": %2$s.', 'key_',
-								_s('%1$d characters exceeds maximum length of %2$d characters', $length, 255)
+								_s('%1$d characters exceeds maximum length of %2$d characters', $length, $max_length)
 							)
 						);
 					}
@@ -543,6 +608,14 @@ class CDRule extends CApiService {
 						self::exception(ZBX_API_ERROR_PARAMETERS, _('Incorrect SNMP OID.'));
 					}
 					break;
+				case SVC_ICMPPING:
+					if (array_key_exists('allow_redirect', $dcheck)
+							&& $dcheck['allow_redirect'] != 1 && $dcheck['allow_redirect'] != 0) {
+						self::exception(ZBX_API_ERROR_PARAMETERS,
+							_s('Incorrect value "%1$s" for "%2$s" field.', $dcheck['allow_redirect'], 'allow_redirect')
+						);
+					}
+					break;
 			}
 
 			// validate snmpv3 fields
@@ -552,8 +625,7 @@ class CDRule extends CApiService {
 				if ($dcheck['snmpv3_securitylevel'] == ITEM_SNMPV3_SECURITYLEVEL_AUTHNOPRIV
 						|| $dcheck['snmpv3_securitylevel'] == ITEM_SNMPV3_SECURITYLEVEL_AUTHPRIV) {
 					if (!array_key_exists('snmpv3_authprotocol', $dcheck)
-							|| $dcheck['snmpv3_authprotocol'] != ITEM_AUTHPROTOCOL_MD5
-								&& $dcheck['snmpv3_authprotocol'] != ITEM_AUTHPROTOCOL_SHA) {
+							|| !array_key_exists($dcheck['snmpv3_authprotocol'], getSnmpV3AuthProtocols())) {
 						self::exception(ZBX_API_ERROR_PARAMETERS,
 							_s('Incorrect value "%1$s" for "%2$s" field.',
 								$dcheck['snmpv3_authprotocol'], 'snmpv3_authprotocol'
@@ -565,8 +637,7 @@ class CDRule extends CApiService {
 				// snmpv3 privprotocol
 				if ($dcheck['snmpv3_securitylevel'] == ITEM_SNMPV3_SECURITYLEVEL_AUTHPRIV) {
 					if (!array_key_exists('snmpv3_privprotocol', $dcheck)
-							|| $dcheck['snmpv3_privprotocol'] != ITEM_PRIVPROTOCOL_DES
-								&& $dcheck['snmpv3_privprotocol'] != ITEM_PRIVPROTOCOL_AES) {
+							|| !array_key_exists($dcheck['snmpv3_privprotocol'], getSnmpV3PrivProtocols())) {
 						self::exception(ZBX_API_ERROR_PARAMETERS,
 							_s('Incorrect value "%1$s" for "%2$s" field.',
 								$dcheck['snmpv3_privprotocol'], 'snmpv3_privprotocol'
@@ -595,31 +666,33 @@ class CDRule extends CApiService {
 
 			switch ($dcheck['snmpv3_securitylevel']) {
 				case ITEM_SNMPV3_SECURITYLEVEL_NOAUTHNOPRIV:
-					$dcheck['snmpv3_authprotocol'] = ITEM_AUTHPROTOCOL_MD5;
-					$dcheck['snmpv3_privprotocol'] = ITEM_PRIVPROTOCOL_DES;
+					$dcheck['snmpv3_authprotocol'] = ITEM_SNMPV3_AUTHPROTOCOL_MD5;
+					$dcheck['snmpv3_privprotocol'] = ITEM_SNMPV3_PRIVPROTOCOL_DES;
 					$dcheck['snmpv3_authpassphrase'] = '';
 					$dcheck['snmpv3_privpassphrase'] = '';
 					break;
 				case ITEM_SNMPV3_SECURITYLEVEL_AUTHNOPRIV:
-					$dcheck['snmpv3_privprotocol'] = ITEM_PRIVPROTOCOL_DES;
+					$dcheck['snmpv3_privprotocol'] = ITEM_SNMPV3_PRIVPROTOCOL_DES;
 					$dcheck['snmpv3_privpassphrase'] = '';
 					break;
 			}
 
 			$dcheck += $default_values;
-			unset($dcheck['uniq']);
+			unset($dcheck['dcheckid'], $dcheck['uniq']);
 		}
 		unset($dcheck);
 
 		while ($current = array_pop($dchecks)) {
 			foreach ($dchecks as $dcheck) {
 				$equal = true;
+
 				foreach ($dcheck as $field => $value) {
 					if (array_key_exists($field, $current) && (strcmp($value, $current[$field]) !== 0)) {
 						$equal = false;
 						break;
 					}
 				}
+
 				if ($equal) {
 					self::exception(ZBX_API_ERROR_PARAMETERS, _('Checks should be unique.'));
 				}
@@ -632,10 +705,11 @@ class CDRule extends CApiService {
 	 *
 	 * @param array(
 	 *  name => string,
-	 *  proxy_hostid => int,
+	 *  proxyid => int,
 	 *  iprange => string,
 	 *  delay => string,
 	 *  status => int,
+	 * 	concurrency_max => int,
 	 *  dchecks => array(
 	 *  	array(
 	 *  		type => int,
@@ -647,6 +721,7 @@ class CDRule extends CApiService {
 	 *  		snmpv3_authpassphrase => string,
 	 *  		snmpv3_privpassphrase => string,
 	 *  		uniq => int,
+	 * 			allow_redirect => int,
 	 *  	), ...
 	 *  )
 	 * ) $drules
@@ -659,7 +734,14 @@ class CDRule extends CApiService {
 
 		$druleids = DB::insert('drules', $drules);
 
+		array_walk($drules, function (&$drule, $index) use ($druleids) {
+			$drule['druleid'] = $druleids[$index];
+		});
+
+		$this->addAuditBulk(CAudit::ACTION_ADD, CAudit::RESOURCE_DISCOVERY_RULE, $drules);
+
 		$create_dchecks = [];
+
 		foreach ($drules as $dnum => $drule) {
 			foreach ($drule['dchecks'] as $dcheck) {
 				$dcheck['druleid'] = $druleids[$dnum];
@@ -678,10 +760,11 @@ class CDRule extends CApiService {
 	 * @param array(
 	 * 	druleid => int,
 	 *  name => string,
-	 *  proxy_hostid => int,
+	 *  proxyid => int,
 	 *  iprange => string,
 	 *  delay => string,
 	 *  status => int,
+	 *  concurrency_max => int,
 	 *  dchecks => array(
 	 *  	array(
 	 * 			dcheckid => int,
@@ -694,6 +777,7 @@ class CDRule extends CApiService {
 	 *  		snmpv3_authpassphrase => string,
 	 *  		snmpv3_privpassphrase => string,
 	 *  		uniq => int,
+	 * 			allow_redirect => int,
 	 *  	), ...
 	 *  )
 	 * ) $drules
@@ -707,10 +791,10 @@ class CDRule extends CApiService {
 		$this->validateUpdate($drules);
 
 		$db_drules = API::DRule()->get([
-			'output' => ['druleid', 'proxy_hostid', 'name', 'iprange', 'delay', 'status'],
+			'output' => ['druleid', 'proxyid', 'name', 'iprange', 'delay', 'status', 'concurrency_max'],
 			'selectDChecks' => ['dcheckid', 'druleid', 'type', 'key_', 'snmp_community', 'ports', 'snmpv3_securityname',
 				'snmpv3_securitylevel', 'snmpv3_authpassphrase', 'snmpv3_privpassphrase', 'uniq', 'snmpv3_authprotocol',
-				'snmpv3_privprotocol', 'snmpv3_contextname'
+				'snmpv3_privprotocol', 'snmpv3_contextname', 'host_source', 'name_source', 'allow_redirect'
 			],
 			'druleids' => $druleids,
 			'editable' => true,
@@ -719,17 +803,11 @@ class CDRule extends CApiService {
 
 		$default_values = DB::getDefaults('dchecks');
 
-		$upd_drules = [];
-
 		foreach ($drules as $drule) {
 			$db_drule = $db_drules[$drule['druleid']];
 
 			// Update drule if it's modified.
 			if (DB::recordModified('drules', $db_drule, $drule)) {
-				if (array_key_exists('delay', $drule) && $db_drule['delay'] != $drule['delay']) {
-					$drule['nextcheck'] = 0;
-				}
-
 				DB::updateByPk('drules', $drule['druleid'], $drule);
 			}
 
@@ -764,76 +842,77 @@ class CDRule extends CApiService {
 			}
 		}
 
+		$this->addAuditBulk(CAudit::ACTION_UPDATE, CAudit::RESOURCE_DISCOVERY_RULE, $drules, $db_drules);
+
 		return ['druleids' => $druleids];
 	}
 
 	/**
-	 * Delete drules.
-	 *
-	 * @param array $dRuleIds
+	 * @param array $druleids
 	 *
 	 * @return array
 	 */
-	public function delete(array $dRuleIds) {
-		$this->validateDelete($dRuleIds);
+	public function delete(array $druleids) {
+		$api_input_rules = ['type' => API_IDS, 'flags' => API_NOT_EMPTY, 'uniq' => true];
 
-		$actionIds = [];
-		$conditionIds = [];
-
-		$dCheckIds = [];
-
-		$dbChecks = DBselect('SELECT dc.dcheckid FROM dchecks dc WHERE '.dbConditionInt('dc.druleid', $dRuleIds));
-
-		while ($dbCheck = DBfetch($dbChecks)) {
-			$dCheckIds[] = $dbCheck['dcheckid'];
+		if (!CApiInputValidator::validate($api_input_rules, $druleids, '/', $error)) {
+			self::exception(ZBX_API_ERROR_PARAMETERS, $error);
 		}
 
-		$dbConditions = DBselect(
-			'SELECT c.conditionid,c.actionid'.
-			' FROM conditions c'.
-			' WHERE (c.conditiontype='.CONDITION_TYPE_DRULE.' AND '.dbConditionString('c.value', $dRuleIds).')'.
-				' OR (c.conditiontype='.CONDITION_TYPE_DCHECK.' AND '.dbConditionString('c.value', $dCheckIds).')'
-		);
+		$db_drules = $this->get([
+			'output' => ['druleid', 'name'],
+			'druleids' => $druleids,
+			'editable' => true,
+			'preservekeys' => true
+		]);
 
-		while ($dbCondition = DBfetch($dbConditions)) {
-			$conditionIds[] = $dbCondition['conditionid'];
-			$actionIds[] = $dbCondition['actionid'];
-		}
-
-		if ($actionIds) {
-			DB::update('actions', [
-				'values' => ['status' => ACTION_STATUS_DISABLED],
-				'where' => ['actionid' => array_unique($actionIds)]
-			]);
-		}
-
-		if ($conditionIds) {
-			DB::delete('conditions', ['conditionid' => $conditionIds]);
-		}
-
-		$result = DB::delete('drules', ['druleid' => $dRuleIds]);
-		if ($result) {
-			foreach ($dRuleIds as $dRuleId) {
-				add_audit(AUDIT_ACTION_DELETE, AUDIT_RESOURCE_DISCOVERY_RULE, '['.$dRuleId.']');
+		foreach ($druleids as $druleid) {
+			if (!array_key_exists($druleid, $db_drules)) {
+				self::exception(ZBX_API_ERROR_PERMISSIONS,
+					_('No permissions to referred object or it does not exist!')
+				);
 			}
 		}
 
-		return ['druleids' => $dRuleIds];
-	}
+		// Check if discovery rules are used in actions.
+		$db_actions = DBselect(
+			'SELECT a.name,c.value'.
+			' FROM actions a,conditions c'.
+			' WHERE a.actionid=c.actionid'.
+				' AND c.conditiontype='.ZBX_CONDITION_TYPE_DRULE.
+				' AND '.dbConditionString('c.value', $druleids),
+			1
+		);
 
-	/**
-	 * Validates the input parameters for the delete() method.
-	 *
-	 * @throws APIException if the input is invalid
-	 *
-	 * @param array $druleIds
-	 */
-	protected function validateDelete(array $druleIds) {
-		if (!$druleIds) {
-			self::exception(ZBX_API_ERROR_PARAMETERS, _('Empty input parameter.'));
+		if ($db_action = DBfetch($db_actions)) {
+			self::exception(ZBX_API_ERROR_PARAMETERS, _s('Discovery rule "%1$s" is used in "%2$s" action.',
+				$db_drules[$db_action['value']]['name'], $db_action['name']
+			));
 		}
 
-		$this->checkDrulePermissions($druleIds);
+		// Check if discovery checks are used in actions.
+		$db_actions = DBselect(
+			'SELECT a.name,dc.druleid'.
+			' FROM actions a,conditions c,dchecks dc'.
+			' WHERE a.actionid=c.actionid'.
+				' AND '.zbx_dbcast_2bigint('c.value').'=dc.dcheckid'.
+				' AND c.conditiontype='.ZBX_CONDITION_TYPE_DCHECK.
+				' AND '.dbConditionString('dc.druleid', $druleids),
+			1
+		);
+
+		if ($db_action = DBfetch($db_actions)) {
+			self::exception(ZBX_API_ERROR_PARAMETERS, _s('Discovery rule "%1$s" is used in "%2$s" action.',
+				$db_drules[$db_action['druleid']]['name'], $db_action['name']
+			));
+		}
+
+		DB::delete('dchecks', ['druleid' => $druleids]);
+		DB::delete('drules', ['druleid' => $druleids]);
+
+		$this->addAuditBulk(CAudit::ACTION_DELETE, CAudit::RESOURCE_DISCOVERY_RULE, $db_drules);
+
+		return ['druleids' => $druleids];
 	}
 
 	/**
@@ -848,10 +927,11 @@ class CDRule extends CApiService {
 		$dbActions = DBselect(
 			'SELECT DISTINCT c.actionid'.
 			' FROM conditions c'.
-			' WHERE c.conditiontype='.CONDITION_TYPE_DCHECK.
+			' WHERE c.conditiontype='.ZBX_CONDITION_TYPE_DCHECK.
 				' AND '.dbConditionString('c.value', $dCheckIds).
 			' ORDER BY c.actionid'
 		);
+
 		while ($dbAction = DBfetch($dbActions)) {
 			$actionIds[] = $dbAction['actionid'];
 		}
@@ -860,11 +940,11 @@ class CDRule extends CApiService {
 		if ($actionIds) {
 			DB::update('actions', [
 				'values' => ['status' => ACTION_STATUS_DISABLED],
-				'where' => ['actionid' => $actionIds],
+				'where' => ['actionid' => $actionIds]
 			]);
 
 			DB::delete('conditions', [
-				'conditiontype' => CONDITION_TYPE_DCHECK,
+				'conditiontype' => ZBX_CONDITION_TYPE_DCHECK,
 				'value' => $dCheckIds
 			]);
 		}
@@ -878,16 +958,22 @@ class CDRule extends CApiService {
 		// Adding Discovery Checks
 		if (!is_null($options['selectDChecks'])) {
 			if ($options['selectDChecks'] != API_OUTPUT_COUNT) {
+				$dchecks = [];
 				$relationMap = $this->createRelationMap($result, 'druleid', 'dcheckid', 'dchecks');
-				$dchecks = API::DCheck()->get([
-					'output' => $options['selectDChecks'],
-					'dcheckids' => $relationMap->getRelatedIds(),
-					'nopermissions' => true,
-					'preservekeys' => true
-				]);
-				if (!is_null($options['limitSelects'])) {
-					order_result($dchecks, 'dcheckid');
+				$related_ids = $relationMap->getRelatedIds();
+
+				if ($related_ids) {
+					$dchecks = API::DCheck()->get([
+						'output' => $options['selectDChecks'],
+						'dcheckids' => $related_ids,
+						'nopermissions' => true,
+						'preservekeys' => true
+					]);
+					if (!is_null($options['limitSelects'])) {
+						order_result($dchecks, 'dcheckid');
+					}
 				}
+
 				$result = $relationMap->mapMany($result, $dchecks, 'dchecks', $options['limitSelects']);
 			}
 			else {
@@ -897,12 +983,13 @@ class CDRule extends CApiService {
 					'countOutput' => true,
 					'groupCount' => true
 				]);
+
 				$dchecks = zbx_toHash($dchecks, 'druleid');
+
 				foreach ($result as $druleid => $drule) {
-					if (isset($dchecks[$druleid]))
-						$result[$druleid]['dchecks'] = $dchecks[$druleid]['rowscount'];
-					else
-						$result[$druleid]['dchecks'] = 0;
+					$result[$druleid]['dchecks'] = array_key_exists($druleid, $dchecks)
+						? $dchecks[$druleid]['rowscount']
+						: '0';
 				}
 			}
 		}
@@ -910,15 +997,22 @@ class CDRule extends CApiService {
 		// Adding Discovery Hosts
 		if (!is_null($options['selectDHosts'])) {
 			if ($options['selectDHosts'] != API_OUTPUT_COUNT) {
+				$dhosts = [];
 				$relationMap = $this->createRelationMap($result, 'druleid', 'dhostid', 'dhosts');
-				$dhosts = API::DHost()->get([
-					'output' => $options['selectDHosts'],
-					'dhostids' => $relationMap->getRelatedIds(),
-					'preservekeys' => true
-				]);
-				if (!is_null($options['limitSelects'])) {
-					order_result($dhosts, 'dhostid');
+				$related_ids = $relationMap->getRelatedIds();
+
+				if ($related_ids) {
+					$dhosts = API::DHost()->get([
+						'output' => $options['selectDHosts'],
+						'dhostids' => $related_ids,
+						'preservekeys' => true
+					]);
+
+					if (!is_null($options['limitSelects'])) {
+						order_result($dhosts, 'dhostid');
+					}
 				}
+
 				$result = $relationMap->mapMany($result, $dhosts, 'dhosts', $options['limitSelects']);
 			}
 			else {
@@ -927,41 +1021,17 @@ class CDRule extends CApiService {
 					'countOutput' => true,
 					'groupCount' => true
 				]);
+
 				$dhosts = zbx_toHash($dhosts, 'druleid');
+
 				foreach ($result as $druleid => $drule) {
-					if (isset($dhosts[$druleid]))
-						$result[$druleid]['dhosts'] = $dhosts[$druleid]['rowscount'];
-					else
-						$result[$druleid]['dhosts'] = 0;
+					$result[$druleid]['dhosts'] = array_key_exists($druleid, $dhosts)
+						? $dhosts[$druleid]['rowscount']
+						: '0';
 				}
 			}
 		}
 
 		return $result;
 	}
-
-	/**
-	 * Checks if the current user has access to given discovery rules.
-	 *
-	 * @throws APIException if the user doesn't have write permissions for discovery rules.
-	 *
-	 * @param array $druleids
-	 */
-	protected function checkDrulePermissions(array $druleids) {
-		if ($druleids) {
-			$druleids = array_unique($druleids);
-
-			$count = $this->get([
-				'countOutput' => true,
-				'druleids' => $druleids,
-				'editable' => true
-			]);
-
-			if ($count != count($druleids)) {
-				self::exception(ZBX_API_ERROR_PERMISSIONS,
-					_('No permissions to referred object or it does not exist!')
-				);
-			}
-		}
-	}
 }
