@@ -1,6 +1,6 @@
 <?php
 /*
-** Copyright (C) 2001-2024 Zabbix SIA
+** Copyright (C) 2001-2025 Zabbix SIA
 **
 ** This program is free software: you can redistribute it and/or modify it under the terms of
 ** the GNU Affero General Public License as published by the Free Software Foundation, version 3.
@@ -134,11 +134,10 @@ class CMap extends CMapElement {
 
 		if ($result) {
 			$result = $this->addRelatedObjects($options, $result);
-		}
 
-		// removing keys (hash -> array)
-		if (!$options['preservekeys']) {
-			$result = zbx_cleanHashes($result);
+			if (!$options['preservekeys']) {
+				$result = array_values($result);
+			}
 		}
 
 		return $result;
@@ -1991,6 +1990,11 @@ class CMap extends CMapElement {
 
 			// Urls.
 			if (array_key_exists('urls', $map)) {
+				foreach ($map['urls'] as &$url) {
+					unset($url['sysmapurlid'], $url['sysmapid']);
+				}
+				unset($url);
+
 				$url_diff = zbx_array_diff($map['urls'], $db_map['urls'], 'name');
 
 				foreach ($url_diff['both'] as $updateUrl) {
