@@ -1,21 +1,16 @@
 <?php
 /*
-** Zabbix
-** Copyright (C) 2001-2023 Zabbix SIA
+** Copyright (C) 2001-2025 Zabbix SIA
 **
-** This program is free software; you can redistribute it and/or modify
-** it under the terms of the GNU General Public License as published by
-** the Free Software Foundation; either version 2 of the License, or
-** (at your option) any later version.
+** This program is free software: you can redistribute it and/or modify it under the terms of
+** the GNU Affero General Public License as published by the Free Software Foundation, version 3.
 **
-** This program is distributed in the hope that it will be useful,
-** but WITHOUT ANY WARRANTY; without even the implied warranty of
-** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-** GNU General Public License for more details.
+** This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
+** without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+** See the GNU Affero General Public License for more details.
 **
-** You should have received a copy of the GNU General Public License
-** along with this program; if not, write to the Free Software
-** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+** You should have received a copy of the GNU Affero General Public License along with this program.
+** If not, see <https://www.gnu.org/licenses/>.
 **/
 
 
@@ -70,7 +65,6 @@ class CTemplate extends CHostGeneral {
 			'searchWildcardsEnabled'	=> null,
 			// output
 			'output'					=> API_OUTPUT_EXTEND,
-			'selectGroups'				=> null,
 			'selectTemplateGroups'		=> null,
 			'selectHosts'				=> null,
 			'selectTemplates'			=> null,
@@ -95,24 +89,21 @@ class CTemplate extends CHostGeneral {
 		$options = zbx_array_merge($defOptions, $options);
 		$this->validateGet($options);
 
-		$this->checkDeprecatedParam($options, 'selectGroups');
-
 		// editable + PERMISSION CHECK
 		if (self::$userData['type'] != USER_TYPE_SUPER_ADMIN && !$options['nopermissions']) {
-			$permission = $options['editable'] ? PERM_READ_WRITE : PERM_READ;
-			$userGroups = getUserGroupsByUserId(self::$userData['userid']);
+			if (self::$userData['ugsetid'] == 0) {
+				return $options['countOutput'] ? '0' : [];
+			}
 
-			$sqlParts['where'][] = 'EXISTS ('.
-					'SELECT NULL'.
-					' FROM hosts_groups hgg'.
-						' JOIN rights r'.
-							' ON r.id=hgg.groupid'.
-								' AND '.dbConditionInt('r.groupid', $userGroups).
-					' WHERE h.hostid=hgg.hostid'.
-					' GROUP BY hgg.hostid'.
-					' HAVING MIN(r.permission)>'.PERM_DENY.
-						' AND MAX(r.permission)>='.zbx_dbstr($permission).
-					')';
+			$sqlParts['from'][] = 'host_hgset hh';
+			$sqlParts['from'][] = 'permission p';
+			$sqlParts['where'][] = 'h.hostid=hh.hostid';
+			$sqlParts['where'][] = 'hh.hgsetid=p.hgsetid';
+			$sqlParts['where'][] = 'p.ugsetid='.self::$userData['ugsetid'];
+
+			if ($options['editable']) {
+				$sqlParts['where'][] = 'p.permission='.PERM_READ_WRITE;
+			}
 		}
 
 		// groupids
@@ -289,11 +280,10 @@ class CTemplate extends CHostGeneral {
 		if ($result) {
 			$result = $this->addRelatedObjects($options, $result);
 			$result = $this->unsetExtraFields($result, ['name_upper']);
-		}
 
-		// removing keys (hash -> array)
-		if (!$options['preservekeys']) {
-			$result = zbx_cleanHashes($result);
+			if (!$options['preservekeys']) {
+				$result = array_values($result);
+			}
 		}
 
 		return $result;
@@ -347,6 +337,7 @@ class CTemplate extends CHostGeneral {
 		$this->checkTemplatesLinks($templates);
 
 		$this->updateGroups($templates);
+		$this->updateHgSets($templates);
 		$this->updateTags($templates);
 		$this->updateMacros($templates);
 		$this->updateTemplates($templates);
@@ -362,7 +353,7 @@ class CTemplate extends CHostGeneral {
 	 * @throws APIException if the input is invalid.
 	 */
 	protected function validateCreate(array &$templates) {
-		$api_input_rules = ['type' => API_OBJECTS, 'flags' => API_NOT_EMPTY | API_NORMALIZE, 'uniq' => [['host'], ['name']], 'fields' => [
+		$api_input_rules = ['type' => API_OBJECTS, 'flags' => API_NOT_EMPTY | API_NORMALIZE, 'uniq' => [['uuid'], ['host'], ['name']], 'fields' => [
 			'uuid' =>			['type' => API_UUID],
 			'host' =>			['type' => API_H_NAME, 'flags' => API_REQUIRED, 'length' => DB::getFieldLength('hosts', 'host')],
 			'name' =>			['type' => API_STRING_UTF8, 'flags' => API_NOT_EMPTY, 'length' => DB::getFieldLength('hosts', 'name'), 'default_source' => 'host'],
@@ -395,36 +386,68 @@ class CTemplate extends CHostGeneral {
 		}
 
 		self::checkVendorFields($templates);
-		self::checkAndAddUuid($templates);
+
+		self::addUuid($templates);
+
+		self::checkUuidDuplicates($templates);
 		$this->checkDuplicates($templates);
 		$this->checkGroups($templates);
 		$this->checkTemplates($templates);
 	}
 
 	/**
-	 * Check that no duplicate UUID is being added. Add UUID to all templates, if it doesn't exist.
+	 * Add the UUID to those of the given templates that don't have the 'uuid' parameter set.
 	 *
-	 * @param array $templates_to_create
-	 *
-	 * @throws APIException
+	 * @param array $templates
 	 */
-	private static function checkAndAddUuid(array &$templates_to_create): void {
-		foreach ($templates_to_create as &$template) {
+	private static function addUuid(array &$templates): void {
+		foreach ($templates as &$template) {
 			if (!array_key_exists('uuid', $template)) {
 				$template['uuid'] = generateUuidV4();
 			}
 		}
 		unset($template);
+	}
 
-		$db_uuid = DB::select('hosts', [
+	/**
+	 * Verify template UUIDs are not repeated.
+	 *
+	 * @param array      $templates
+	 * @param array|null $db_templates
+	 *
+	 * @throws APIException
+	 */
+	private static function checkUuidDuplicates(array $templates, array $db_templates = null): void {
+		$template_indexes = [];
+
+		foreach ($templates as $i => $template) {
+			if (!array_key_exists('uuid', $template)) {
+				continue;
+			}
+
+			if ($db_templates === null || $template['uuid'] !== $db_templates[$template['templateid']]['uuid']) {
+				$template_indexes[$template['uuid']] = $i;
+			}
+		}
+
+		if (!$template_indexes) {
+			return;
+		}
+
+		$duplicates = DB::select('hosts', [
 			'output' => ['uuid'],
-			'filter' => ['uuid' => array_column($templates_to_create, 'uuid')],
+			'filter' => [
+				'status' => HOST_STATUS_TEMPLATE,
+				'uuid' => array_keys($template_indexes)
+			],
 			'limit' => 1
 		]);
 
-		if ($db_uuid) {
+		if ($duplicates) {
 			self::exception(ZBX_API_ERROR_PARAMETERS,
-				_s('Entry with UUID "%1$s" already exists.', $db_uuid[0]['uuid'])
+				_s('Invalid parameter "%1$s": %2$s.', '/'.($template_indexes[$duplicates[0]['uuid']] + 1),
+					_('template with the same UUID already exists')
+				)
 			);
 		}
 	}
@@ -436,30 +459,7 @@ class CTemplate extends CHostGeneral {
 	 */
 	public function update(array $templates): array {
 		$this->validateUpdate($templates, $db_templates);
-
-		$upd_templates =[];
-
-		foreach ($templates as $template) {
-			$upd_template = DB::getUpdatedValues('hosts', $template, $db_templates[$template['templateid']]);
-
-			if ($upd_template) {
-				$upd_templates[] = [
-					'values' => $upd_template,
-					'where' => ['hostid' => $template['templateid']]
-				];
-			}
-		}
-
-		if ($upd_templates) {
-			DB::update('hosts', $upd_templates);
-		}
-
-		$this->updateGroups($templates, $db_templates);
-		$this->updateTags($templates, $db_templates);
-		$this->updateMacros($templates, $db_templates);
-		$this->updateTemplates($templates, $db_templates);
-
-		self::addAuditLog(CAudit::ACTION_UPDATE, CAudit::RESOURCE_TEMPLATE, $templates, $db_templates);
+		$this->updateForce($templates, $db_templates);
 
 		return ['templateids' => array_column($templates, 'templateid')];
 	}
@@ -471,7 +471,7 @@ class CTemplate extends CHostGeneral {
 	 * @throws APIException if the input is invalid.
 	 */
 	protected function validateUpdate(array &$templates, array &$db_templates = null) {
-		$api_input_rules = ['type' => API_OBJECTS, 'flags' => API_NOT_EMPTY | API_NORMALIZE, 'uniq' => [['templateid'], ['host'], ['name']], 'fields' => [
+		$api_input_rules = ['type' => API_OBJECTS, 'flags' => API_NOT_EMPTY | API_NORMALIZE, 'uniq' => [['uuid'], ['templateid'], ['host'], ['name']], 'fields' => [
 			'uuid' => 				['type' => API_UUID],
 			'templateid' =>			['type' => API_ID, 'flags' => API_REQUIRED],
 			'host' =>				['type' => API_H_NAME, 'length' => DB::getFieldLength('hosts', 'host')],
@@ -506,7 +506,7 @@ class CTemplate extends CHostGeneral {
 		}
 
 		$db_templates = $this->get([
-			'output' => ['templateid', 'host', 'name', 'description', 'vendor_name', 'vendor_version'],
+			'output' => ['uuid', 'templateid', 'host', 'name', 'description', 'vendor_name', 'vendor_version'],
 			'templateids' => array_column($templates, 'templateid'),
 			'editable' => true,
 			'preservekeys' => true
@@ -519,6 +519,7 @@ class CTemplate extends CHostGeneral {
 		$this->addAffectedObjects($templates, $db_templates);
 
 		self::checkVendorFields($templates, $db_templates);
+		self::checkUuidDuplicates($templates, $db_templates);
 		$this->checkDuplicates($templates, $db_templates);
 		$this->checkGroups($templates, $db_templates);
 		$this->checkTemplates($templates, $db_templates);
@@ -559,6 +560,33 @@ class CTemplate extends CHostGeneral {
 		}
 	}
 
+	public function updateForce(array $templates, array $db_templates): void {
+		$upd_templates =[];
+
+		foreach ($templates as $template) {
+			$upd_template = DB::getUpdatedValues('hosts', $template, $db_templates[$template['templateid']]);
+
+			if ($upd_template) {
+				$upd_templates[] = [
+					'values' => $upd_template,
+					'where' => ['hostid' => $template['templateid']]
+				];
+			}
+		}
+
+		if ($upd_templates) {
+			DB::update('hosts', $upd_templates);
+		}
+
+		$this->updateGroups($templates, $db_templates);
+		$this->updateHgSets($templates, $db_templates);
+		$this->updateTags($templates, $db_templates);
+		$this->updateMacros($templates, $db_templates);
+		$this->updateTemplates($templates, $db_templates);
+
+		self::addAuditLog(CAudit::ACTION_UPDATE, CAudit::RESOURCE_TEMPLATE, $templates, $db_templates);
+	}
+
 	/**
 	 * Delete template.
 	 *
@@ -573,15 +601,16 @@ class CTemplate extends CHostGeneral {
 		self::unlinkTemplatesObjects($templateids, null, true);
 
 		// delete the discovery rules first
-		$del_rules = API::DiscoveryRule()->get([
-			'output' => [],
-			'hostids' => $templateids,
-			'nopermissions' => true,
+		$db_lld_rules = DB::select('items', [
+			'output' => ['itemid', 'name'],
+			'filter' => [
+				'hostid' => $templateids,
+				'flags' => ZBX_FLAG_DISCOVERY_RULE
+			],
 			'preservekeys' => true
 		]);
-
-		if ($del_rules) {
-			CDiscoveryRuleManager::delete(array_keys($del_rules));
+		if ($db_lld_rules) {
+			CDiscoveryRule::deleteForce($db_lld_rules);
 		}
 
 		// delete the items
@@ -609,7 +638,7 @@ class CTemplate extends CHostGeneral {
 		$actionids = [];
 		$sql = 'SELECT DISTINCT actionid'.
 			' FROM conditions'.
-			' WHERE conditiontype='.CONDITION_TYPE_TEMPLATE.
+			' WHERE conditiontype='.ZBX_CONDITION_TYPE_TEMPLATE.
 			' AND '.dbConditionString('value', $templateids);
 		$dbActions = DBselect($sql);
 		while ($dbAction = DBfetch($dbActions)) {
@@ -635,7 +664,7 @@ class CTemplate extends CHostGeneral {
 
 		// delete action conditions
 		DB::delete('conditions', [
-			'conditiontype' => CONDITION_TYPE_TEMPLATE,
+			'conditiontype' => ZBX_CONDITION_TYPE_TEMPLATE,
 			'value' => $templateids
 		]);
 
@@ -737,6 +766,9 @@ class CTemplate extends CHostGeneral {
 			}
 		}
 
+		self::deleteHgSets($db_templates);
+		DB::delete('hosts_groups', ['hostid' => $templateids]);
+
 		// Finally delete the template.
 		DB::delete('host_tag', ['hostid' => $templateids]);
 		DB::delete('hosts', ['hostid' => $templateids]);
@@ -816,19 +848,78 @@ class CTemplate extends CHostGeneral {
 	 * @return array
 	 */
 	public function massAdd(array $data) {
-		$this->validateMassAdd($data, $db_templates);
+		$this->validateMassAdd($data, $templates, $db_templates);
 
-		$templates = $this->getObjectsByData($data, $db_templates);
-
-		$this->updateGroups($templates, $db_templates);
-		$this->updateMacros($templates, $db_templates);
-		$this->updateTemplates($templates, $db_templates);
-
-		self::addAuditLog(CAudit::ACTION_UPDATE, CAudit::RESOURCE_TEMPLATE, $templates, $db_templates);
+		$this->updateForce($templates, $db_templates);
 
 		return ['templateids' => array_column($data['templates'], 'templateid')];
 	}
 
+	private function validateMassAdd(array &$data, ?array &$templates, ?array &$db_templates): void {
+		$api_input_rules = ['type' => API_OBJECT, 'fields' => [
+			'templates' =>			['type' => API_OBJECTS, 'flags' => API_REQUIRED | API_NOT_EMPTY | API_NORMALIZE, 'uniq' => [['templateid']], 'fields' => [
+				'templateid' =>			['type' => API_ID, 'flags' => API_REQUIRED]
+			]],
+			'groups' =>				['type' => API_OBJECTS, 'flags' => API_NORMALIZE, 'uniq' => [['groupid']], 'fields' => [
+				'groupid' =>			['type' => API_ID, 'flags' => API_REQUIRED]
+			]],
+			'macros' =>				['type' => API_OBJECTS, 'flags' => API_NORMALIZE, 'uniq' => [['macro']], 'fields' => [
+				'macro' =>				['type' => API_USER_MACRO, 'flags' => API_REQUIRED, 'length' => DB::getFieldLength('hostmacro', 'macro')],
+				'type' =>				['type' => API_INT32, 'in' => implode(',', [ZBX_MACRO_TYPE_TEXT, ZBX_MACRO_TYPE_SECRET, ZBX_MACRO_TYPE_VAULT]), 'default' => ZBX_MACRO_TYPE_TEXT],
+				'value' =>				['type' => API_MULTIPLE, 'flags' => API_REQUIRED, 'rules' => [
+											['if' => ['field' => 'type', 'in' => implode(',', [ZBX_MACRO_TYPE_VAULT])], 'type' => API_VAULT_SECRET, 'provider' => CSettingsHelper::get(CSettingsHelper::VAULT_PROVIDER), 'length' => DB::getFieldLength('hostmacro', 'value')],
+											['else' => true, 'type' => API_STRING_UTF8, 'length' => DB::getFieldLength('hostmacro', 'value')]
+				]],
+				'description' =>		['type' => API_STRING_UTF8, 'length' => DB::getFieldLength('hostmacro', 'description')]
+			]],
+			'templates_link' =>		['type' => API_OBJECTS, 'flags' => API_NORMALIZE, 'uniq' => [['templateid']], 'fields' => [
+				'templateid' =>			['type' => API_ID, 'flags' => API_REQUIRED]
+			]]
+		]];
+
+		if (!CApiInputValidator::validate($api_input_rules, $data, '/', $error)) {
+			self::exception(ZBX_API_ERROR_PARAMETERS, $error);
+		}
+
+		$db_templates = $this->get([
+			'output' => ['templateid', 'host'],
+			'templateids' => array_column($data['templates'], 'templateid'),
+			'editable' => true,
+			'preservekeys' => true
+		]);
+
+		foreach ($data['templates'] as $i => $template) {
+			if (!array_key_exists($template['templateid'], $db_templates)) {
+				self::exception(ZBX_API_ERROR_PERMISSIONS, _s('Invalid parameter "%1$s": %2$s.',
+					'/templates/'.($i + 1), _('object does not exist, or you have no permissions to it')
+				));
+			}
+		}
+
+		$templates = $data['templates'];
+
+		$this->addObjectsByData($data, $templates);
+		$this->addAffectedObjects($templates, $db_templates);
+		$this->addUnchangedObjects($templates, $db_templates);
+
+		if (array_key_exists('groups', $data) && $data['groups']) {
+			$this->checkGroups($templates, $db_templates, '/groups',
+				array_flip(array_column($data['groups'], 'groupid'))
+			);
+		}
+
+		if (array_key_exists('macros', $data) && $data['macros']) {
+			$templates = $this->validateHostMacros($templates, $db_templates);
+		}
+
+		if (array_key_exists('templates_link', $data) && $data['templates_link']) {
+			$this->checkTemplates($templates, $db_templates, '/templates_link',
+				array_flip(array_column($data['templates_link'], 'templateid'))
+			);
+			$this->checkTemplatesLinks($templates, $db_templates);
+		}
+	}
+
 	/**
 	 * Replace template groups, macros and templates on the given templates.
 	 *
@@ -837,19 +928,94 @@ class CTemplate extends CHostGeneral {
 	 * @return array
 	 */
 	public function massUpdate(array $data) {
-		$this->validateMassUpdate($data, $db_templates);
+		$this->validateMassUpdate($data, $templates, $db_templates);
 
-		$templates = $this->getObjectsByData($data, $db_templates);
-
-		$this->updateGroups($templates, $db_templates);
-		$this->updateMacros($templates, $db_templates);
-		$this->updateTemplates($templates, $db_templates);
-
-		self::addAuditLog(CAudit::ACTION_UPDATE, CAudit::RESOURCE_TEMPLATE, $templates, $db_templates);
+		$this->updateForce($templates, $db_templates);
 
 		return ['templateids' => array_column($data['templates'], 'templateid')];
 	}
 
+	private function validateMassUpdate(array &$data, ?array &$templates, ?array &$db_templates): void {
+		$api_input_rules = ['type' => API_OBJECT, 'fields' => [
+			'templates' =>			['type' => API_OBJECTS, 'flags' => API_REQUIRED | API_NOT_EMPTY | API_NORMALIZE, 'uniq' => [['templateid']], 'fields' => [
+				'templateid' =>			['type' => API_ID, 'flags' => API_REQUIRED]
+			]],
+			'groups' =>				['type' => API_OBJECTS, 'flags' => API_NOT_EMPTY | API_NORMALIZE, 'uniq' => [['groupid']], 'fields' => [
+				'groupid' =>			['type' => API_ID, 'flags' => API_REQUIRED]
+			]],
+			'macros' =>				['type' => API_OBJECTS, 'flags' => API_NORMALIZE, 'uniq' => [['macro']], 'fields' => [
+				'macro' =>				['type' => API_USER_MACRO, 'flags' => API_REQUIRED, 'length' => DB::getFieldLength('hostmacro', 'macro')],
+				'type' =>				['type' => API_INT32, 'in' => implode(',', [ZBX_MACRO_TYPE_TEXT, ZBX_MACRO_TYPE_SECRET, ZBX_MACRO_TYPE_VAULT]), 'default' => ZBX_MACRO_TYPE_TEXT],
+				'value' =>				['type' => API_MULTIPLE, 'flags' => API_REQUIRED, 'rules' => [
+											['if' => ['field' => 'type', 'in' => implode(',', [ZBX_MACRO_TYPE_VAULT])], 'type' => API_VAULT_SECRET, 'provider' => CSettingsHelper::get(CSettingsHelper::VAULT_PROVIDER), 'length' => DB::getFieldLength('hostmacro', 'value')],
+											['else' => true, 'type' => API_STRING_UTF8, 'length' => DB::getFieldLength('hostmacro', 'value')]
+				]],
+				'description' =>		['type' => API_STRING_UTF8, 'length' => DB::getFieldLength('hostmacro', 'description')]
+			]],
+			'templates_link' =>		['type' => API_OBJECTS, 'flags' => API_NORMALIZE, 'uniq' => [['templateid']], 'fields' => [
+				'templateid' =>			['type' => API_ID, 'flags' => API_REQUIRED]
+			]],
+			'templates_clear' =>	['type' => API_OBJECTS, 'flags' => API_NORMALIZE, 'uniq' => [['templateid']], 'fields' => [
+				'templateid' =>			['type' => API_ID, 'flags' => API_REQUIRED]
+			]]
+		]];
+
+		if (!CApiInputValidator::validate($api_input_rules, $data, '/', $error)) {
+			self::exception(ZBX_API_ERROR_PARAMETERS, $error);
+		}
+
+		$db_templates = $this->get([
+			'output' => ['templateid', 'host'],
+			'templateids' => array_column($data['templates'], 'templateid'),
+			'editable' => true,
+			'preservekeys' => true
+		]);
+
+		foreach ($data['templates'] as $i => $template) {
+			if (!array_key_exists($template['templateid'], $db_templates)) {
+				self::exception(ZBX_API_ERROR_PERMISSIONS, _s('Invalid parameter "%1$s": %2$s.',
+					'/templates/'.($i + 1), _('object does not exist, or you have no permissions to it')
+				));
+			}
+		}
+
+		$templates = $data['templates'];
+
+		$this->addObjectsByData($data, $templates);
+		$this->addAffectedObjects($templates, $db_templates);
+
+		if (array_key_exists('groups', $data)) {
+			$this->checkGroups($templates, $db_templates, '/groups',
+				array_flip(array_column($data['groups'], 'groupid'))
+			);
+			$this->checkHostsWithoutGroups($templates, $db_templates);
+		}
+
+		if (array_key_exists('macros', $data) && $data['macros']) {
+			self::addHostMacroIds($templates, $db_templates);
+		}
+
+		if (array_key_exists('templates_link', $data)
+				|| (array_key_exists('templates_clear', $data) && $data['templates_clear'])) {
+			$path = array_key_exists('templates_link', $data) ? '/templates_link' : null;
+			$template_indexes = array_key_exists('templates_link', $data)
+				? array_flip(array_column($data['templates_link'], 'templateid'))
+				: null;
+
+			$path_clear = array_key_exists('templates_clear', $data) && $data['templates_clear']
+				? '/templates_clear'
+				: null;
+			$template_clear_indexes = array_key_exists('templates_clear', $data) && $data['templates_clear']
+				? array_flip(array_column($data['templates_clear'], 'templateid'))
+				: null;
+
+			$this->checkTemplates($templates, $db_templates, $path, $template_indexes, $path_clear,
+				$template_clear_indexes
+			);
+			$this->checkTemplatesLinks($templates, $db_templates);
+		}
+	}
+
 	/**
 	 * Remove given template groups, macros and templates from given templates.
 	 *
@@ -858,297 +1024,14 @@ class CTemplate extends CHostGeneral {
 	 * @return array
 	 */
 	public function massRemove(array $data) {
-		$this->validateMassRemove($data, $db_templates);
+		$this->validateMassRemove($data, $templates, $db_templates);
 
-		$templates = $this->getObjectsByData($data, $db_templates);
-
-		$this->updateGroups($templates, $db_templates);
-		$this->updateMacros($templates, $db_templates);
-		$this->updateTemplates($templates, $db_templates);
-
-		self::addAuditLog(CAudit::ACTION_UPDATE, CAudit::RESOURCE_TEMPLATE, $templates, $db_templates);
+		$this->updateForce($templates, $db_templates);
 
 		return ['templateids' => $data['templateids']];
 	}
 
-	/**
-	 * @param array      $data
-	 * @param array|null $db_templates
-	 *
-	 * @throws APIException if the input is invalid.
-	 */
-	private function validateMassAdd(array &$data, ?array &$db_templates): void {
-		$api_input_rules = ['type' => API_OBJECT, 'fields' => [
-			'templates' =>		['type' => API_OBJECTS, 'flags' => API_REQUIRED | API_NOT_EMPTY | API_NORMALIZE, 'uniq' => [['templateid']], 'fields' => [
-				'templateid' =>		['type' => API_ID, 'flags' => API_REQUIRED]
-			]],
-			'groups' =>			['type' => API_OBJECTS, 'flags' => API_NORMALIZE, 'uniq' => [['groupid']], 'fields' => [
-				'groupid' =>		['type' => API_ID, 'flags' => API_REQUIRED]
-			]],
-			'macros' =>			['type' => API_OBJECTS, 'flags' => API_NORMALIZE, 'uniq' => [['macro']], 'fields' => [
-				'macro' =>			['type' => API_USER_MACRO, 'flags' => API_REQUIRED, 'length' => DB::getFieldLength('hostmacro', 'macro')],
-				'type' =>			['type' => API_INT32, 'in' => implode(',', [ZBX_MACRO_TYPE_TEXT, ZBX_MACRO_TYPE_SECRET, ZBX_MACRO_TYPE_VAULT]), 'default' => ZBX_MACRO_TYPE_TEXT],
-				'value' =>			['type' => API_MULTIPLE, 'flags' => API_REQUIRED, 'rules' => [
-										['if' => ['field' => 'type', 'in' => implode(',', [ZBX_MACRO_TYPE_VAULT])], 'type' => API_VAULT_SECRET, 'provider' => CSettingsHelper::get(CSettingsHelper::VAULT_PROVIDER), 'length' => DB::getFieldLength('hostmacro', 'value')],
-										['else' => true, 'type' => API_STRING_UTF8, 'length' => DB::getFieldLength('hostmacro', 'value')]
-				]],
-				'description' =>	['type' => API_STRING_UTF8, 'length' => DB::getFieldLength('hostmacro', 'description')]
-			]],
-			'templates_link' =>	['type' => API_OBJECTS, 'flags' => API_NORMALIZE, 'uniq' => [['templateid']], 'fields' => [
-				'templateid' =>		['type' => API_ID, 'flags' => API_REQUIRED]
-			]]
-		]];
-
-		if (!CApiInputValidator::validate($api_input_rules, $data, '/', $error)) {
-			self::exception(ZBX_API_ERROR_PARAMETERS, $error);
-		}
-
-		$db_templates = $this->get([
-			'output' => ['templateid', 'host'],
-			'templateids' => array_column($data['templates'], 'templateid'),
-			'editable' => true,
-			'preservekeys' => true
-		]);
-
-		if (count($db_templates) != count($data['templates'])) {
-			self::exception(ZBX_API_ERROR_PERMISSIONS, _('No permissions to referred object or it does not exist!'));
-		}
-
-		if (array_key_exists('groups', $data) && $data['groups']) {
-			$groupids = array_column($data['groups'], 'groupid');
-
-			$count = API::TemplateGroup()->get([
-				'countOutput' => true,
-				'groupids' => $groupids,
-				'editable' => true
-			]);
-
-			if ($count != count($groupids)) {
-				self::exception(ZBX_API_ERROR_PERMISSIONS,
-					_('No permissions to referred object or it does not exist!')
-				);
-			}
-
-			$this->massAddAffectedObjects('groups', $groupids, $db_templates);
-		}
-
-		if (array_key_exists('macros', $data) && $data['macros']) {
-			$macros = [];
-
-			foreach ($data['macros'] as $macro) {
-				$macros[CApiInputValidator::trimMacro($macro['macro'])] = $macro['macro'];
-			}
-
-			$options = [
-				'output' => ['hostid', 'macro'],
-				'filter' => ['hostid' => array_keys($db_templates)]
-			];
-			$db_macros = DBselect(DB::makeSql('hostmacro', $options));
-
-			while ($db_macro = DBfetch($db_macros)) {
-				$trimmed_db_macro = CApiInputValidator::trimMacro($db_macro['macro']);
-
-				if (array_key_exists($trimmed_db_macro, $macros)) {
-					self::exception(ZBX_API_ERROR_PARAMETERS,
-						_s('Macro "%1$s" already exists on "%2$s".', $macros[$trimmed_db_macro],
-							$db_templates[$db_macro['hostid']]['host']
-						)
-					);
-				}
-			}
-
-			foreach ($db_templates as &$db_template) {
-				$db_template['macros'] = [];
-			}
-			unset($db_host);
-		}
-
-		if (array_key_exists('templates_link', $data) && $data['templates_link']) {
-			$templateids = array_column($data['templates_link'], 'templateid');
-
-			$count = API::Template()->get([
-				'countOutput' => true,
-				'templateids' => $templateids
-			]);
-
-			if ($count != count($templateids)) {
-				self::exception(ZBX_API_ERROR_PERMISSIONS,
-					_('No permissions to referred object or it does not exist!')
-				);
-			}
-
-			$this->massAddAffectedObjects('templates', $templateids, $db_templates);
-
-			$this->massCheckTemplatesLinks('massadd', $templateids, $db_templates);
-		}
-	}
-
-	/**
-	 * @param array      $data
-	 * @param array|null $db_templates
-	 *
-	 * @throws APIException if the input is invalid.
-	 */
-	private function validateMassUpdate(array &$data, ?array &$db_templates): void {
-		$api_input_rules = ['type' => API_OBJECT, 'fields' => [
-			'templates' =>		['type' => API_OBJECTS, 'flags' => API_REQUIRED | API_NOT_EMPTY | API_NORMALIZE, 'uniq' => [['templateid']], 'fields' => [
-				'templateid' =>		['type' => API_ID, 'flags' => API_REQUIRED]
-			]],
-			'groups' =>			['type' => API_OBJECTS, 'flags' => API_NOT_EMPTY | API_NORMALIZE, 'uniq' => [['groupid']], 'fields' => [
-				'groupid' =>		['type' => API_ID, 'flags' => API_REQUIRED]
-			]],
-			'macros' =>			['type' => API_OBJECTS, 'flags' => API_NORMALIZE, 'uniq' => [['macro']], 'fields' => [
-				'macro' =>			['type' => API_USER_MACRO, 'flags' => API_REQUIRED, 'length' => DB::getFieldLength('hostmacro', 'macro')],
-				'type' =>			['type' => API_INT32, 'in' => implode(',', [ZBX_MACRO_TYPE_TEXT, ZBX_MACRO_TYPE_SECRET, ZBX_MACRO_TYPE_VAULT]), 'default' => ZBX_MACRO_TYPE_TEXT],
-				'value' =>			['type' => API_MULTIPLE, 'flags' => API_REQUIRED, 'rules' => [
-										['if' => ['field' => 'type', 'in' => implode(',', [ZBX_MACRO_TYPE_VAULT])], 'type' => API_VAULT_SECRET, 'provider' => CSettingsHelper::get(CSettingsHelper::VAULT_PROVIDER), 'length' => DB::getFieldLength('hostmacro', 'value')],
-										['else' => true, 'type' => API_STRING_UTF8, 'length' => DB::getFieldLength('hostmacro', 'value')]
-				]],
-				'description' =>	['type' => API_STRING_UTF8, 'length' => DB::getFieldLength('hostmacro', 'description')]
-			]],
-			'templates_link' =>	['type' => API_OBJECTS, 'flags' => API_NORMALIZE, 'uniq' => [['templateid']], 'fields' => [
-				'templateid' =>		['type' => API_ID, 'flags' => API_REQUIRED]
-			]],
-			'templates_clear' =>	['type' => API_OBJECTS, 'flags' => API_NORMALIZE, 'uniq' => [['templateid']], 'fields' => [
-				'templateid' =>		['type' => API_ID, 'flags' => API_REQUIRED]
-			]]
-		]];
-
-		if (!CApiInputValidator::validate($api_input_rules, $data, '/', $error)) {
-			self::exception(ZBX_API_ERROR_PARAMETERS, $error);
-		}
-
-		$db_templates = $this->get([
-			'output' => ['templateid', 'host'],
-			'templateids' => array_column($data['templates'], 'templateid'),
-			'editable' => true,
-			'preservekeys' => true
-		]);
-
-		if (count($db_templates) != count($data['templates'])) {
-			self::exception(ZBX_API_ERROR_PERMISSIONS, _('No permissions to referred object or it does not exist!'));
-		}
-
-		if (array_key_exists('groups', $data)) {
-			$groupids = array_column($data['groups'], 'groupid');
-
-			$count = API::TemplateGroup()->get([
-				'countOutput' => true,
-				'groupids' => $groupids
-			]);
-
-			if ($count != count($groupids)) {
-				self::exception(ZBX_API_ERROR_PERMISSIONS,
-					_('No permissions to referred object or it does not exist!')
-				);
-			}
-
-			$this->massAddAffectedObjects('groups', [], $db_templates);
-
-			$groupids = array_flip($groupids);
-			$edit_groupids = [];
-
-			foreach ($db_templates as $db_template) {
-				$_groupids = $groupids;
-
-				foreach ($db_template['groups'] as $db_group) {
-					if (array_key_exists($db_group['groupid'], $_groupids)) {
-						unset($_groupids[$db_group['groupid']]);
-					} else {
-						$edit_groupids[$db_group['groupid']] = true;
-					}
-				}
-
-				$edit_groupids += $_groupids;
-			}
-
-			if ($edit_groupids) {
-				$count = API::TemplateGroup()->get([
-					'countOutput' => true,
-					'groupids' => array_keys($edit_groupids),
-					'editable' => true
-				]);
-
-				if ($count != count($edit_groupids)) {
-					self::exception(ZBX_API_ERROR_PERMISSIONS,
-						_('No permissions to referred object or it does not exist!')
-					);
-				}
-			}
-		}
-
-		if (array_key_exists('macros', $data)) {
-			$this->massAddAffectedObjects('macros', [], $db_templates);
-		}
-
-		if (array_key_exists('templates_link', $data)
-				|| (array_key_exists('templates_clear', $data) && $data['templates_clear'])) {
-			if (array_key_exists('templates_link', $data) && array_key_exists('templates_clear', $data)) {
-				$path_clear = '/templates_clear';
-				$path = '/templates_link';
-
-				foreach ($data['templates_clear'] as $i1_clear => $template_clear) {
-					foreach ($data['templates_link'] as $i1 => $template) {
-						if (bccomp($template['templateid'], $template_clear['templateid']) == 0) {
-							self::exception(ZBX_API_ERROR_PARAMETERS, _s('Invalid parameter "%1$s": %2$s.',
-								$path_clear.'/'.($i1_clear + 1).'/templateid',
-								_s('cannot be specified the value of parameter "%1$s"',
-									$path.'/'.($i1 + 1).'/templateid'
-								)
-							));
-						}
-					}
-				}
-			}
-
-			$this->massAddAffectedObjects('templates', [], $db_templates);
-
-			$templateids_link = array_key_exists('templates_link', $data)
-				? array_column($data['templates_link'], 'templateid')
-				: [];
-			$templateids_clear = array_key_exists('templates_clear', $data)
-				? array_column($data['templates_clear'], 'templateid')
-				: [];
-
-			$edit_templateids = array_flip($templateids_clear);
-
-			if ($templateids_link) {
-				foreach ($db_templates as $db_template) {
-					$edit_templateids += array_flip(array_diff(array_column($db_template['templates'], 'templateid'),
-						$templateids_link
-					));
-				}
-			}
-
-			if ($edit_templateids) {
-				$count = $this->get([
-					'countOutput' => true,
-					'templateids' => array_keys($edit_templateids)
-				]);
-
-				if ($count != count($edit_templateids)) {
-					self::exception(ZBX_API_ERROR_PERMISSIONS,
-						_('No permissions to referred object or it does not exist!')
-					);
-				}
-
-				if (array_key_exists('templates_link', $data)) {
-					$this->massCheckTemplatesLinks('massupdate', $templateids_link, $db_templates, $templateids_clear);
-				}
-				else {
-					$this->massCheckTemplatesLinks('massremove', $templateids_clear, $db_templates, $templateids_clear);
-				}
-			}
-		}
-	}
-
-	/**
-	 * @param array      $data
-	 * @param array|null $db_templates
-	 *
-	 * @throws APIException if the input is invalid.
-	 */
-	private function validateMassRemove(array &$data, ?array &$db_templates): void {
+	private function validateMassRemove(array &$data, ?array &$templates, ?array &$db_templates): void {
 		$api_input_rules = ['type' => API_OBJECT, 'fields' => [
 			'templateids' =>		['type' => API_IDS, 'flags' => API_REQUIRED | API_NOT_EMPTY | API_NORMALIZE, 'uniq' => true],
 			'groupids' =>			['type' => API_IDS, 'flags' => API_NORMALIZE, 'uniq' => true],
@@ -1168,70 +1051,62 @@ class CTemplate extends CHostGeneral {
 			'preservekeys' => true
 		]);
 
-		if (count($db_templates) != count($data['templateids'])) {
-			self::exception(ZBX_API_ERROR_PERMISSIONS, _('No permissions to referred object or it does not exist!'));
+		foreach ($data['templateids'] as $i => $templateid) {
+			if (!array_key_exists($templateid, $db_templates)) {
+				self::exception(ZBX_API_ERROR_PERMISSIONS, _s('Invalid parameter "%1$s": %2$s.',
+					'/templateids/'.($i + 1), _('object does not exist, or you have no permissions to it')
+				));
+			}
 		}
 
+		$templates = [];
+
+		foreach ($data['templateids'] as $templateid) {
+			$templates[] = ['templateid' => $templateid];
+		}
+
+		$data = CArrayHelper::renameKeys($data, ['macros' => 'macro_names']);
+
+		$this->addObjectsByData($data, $templates);
+		$this->addAffectedObjects($templates, $db_templates);
+		$this->addUnchangedObjects($templates, $db_templates, $data);
+
 		if (array_key_exists('groupids', $data) && $data['groupids']) {
-			$count = API::TemplateGroup()->get([
-				'countOutput' => true,
-				'groupids' => $data['groupids'],
-				'editable' => true
-			]);
-
-			if ($count != count($data['groupids'])) {
-				self::exception(ZBX_API_ERROR_PERMISSIONS,
-					_('No permissions to referred object or it does not exist!')
-				);
-			}
-
-			CTemplateGroup::checkTemplatesWithoutGroups($db_templates, $data['groupids']);
-
-			$this->massAddAffectedObjects('groups', $data['groupids'], $db_templates);
-		}
-
-		if (array_key_exists('macros', $data) && $data['macros']) {
-			$this->massAddAffectedObjects('macros', $data['macros'], $db_templates);
+			$this->checkGroups($templates, $db_templates, '/groupids', array_flip($data['groupids']));
+			$this->checkHostsWithoutGroups($templates, $db_templates);
 		}
 
 		if ((array_key_exists('templateids_link', $data) && $data['templateids_link'])
 				|| (array_key_exists('templateids_clear', $data) && $data['templateids_clear'])) {
-			if (array_key_exists('templateids_link', $data) && $data['templateids_link']
-					&& array_key_exists('templateids_clear', $data) && $data['templateids_clear']) {
-				$templateids = array_unique(array_merge($data['templateids_link'], $data['templateids_clear']));
-			}
-			elseif (array_key_exists('templateids_link', $data) && $data['templateids_link']) {
-				$templateids = $data['templateids_link'];
-			}
-			else {
-				$templateids = $data['templateids_clear'];
-			}
+			$path_clear = array_key_exists('templateids_clear', $data) && $data['templateids_clear']
+				? '/templateids_clear'
+				: null;
+			$template_clear_indexes = array_key_exists('templateids_clear', $data) && $data['templateids_clear']
+				? array_flip($data['templateids_clear'])
+				: null;
 
-			$count = $this->get([
-				'countOutput' => true,
-				'templateids' => $templateids
-			]);
-
-			if ($count != count($templateids)) {
-				self::exception(ZBX_API_ERROR_PERMISSIONS,
-					_('No permissions to referred object or it does not exist!')
-				);
-			}
-
-			$this->massAddAffectedObjects('templates', $templateids, $db_templates);
-
-			$this->massCheckTemplatesLinks('massremove', $templateids, $db_templates,
-				array_key_exists('templateids_clear', $data) ? $data['templateids_clear'] : []
-			);
+			$this->checkTemplates($templates, $db_templates, null, null, $path_clear, $template_clear_indexes);
+			$this->checkTemplatesLinks($templates, $db_templates);
 		}
 	}
 
+	private function addObjectsByData(array $data, array &$templates): void {
+		self::addGroupsByData($data, $templates);
+		self::addMacrosByData($data, $templates);
+		$this->addTemplatesByData($data, $templates);
+		self::addTemplatesClearByData($data, $templates);
+	}
+
+	private function addUnchangedObjects(array &$templates, array $db_templates, array $del_objectids = []): void {
+		$this->addUnchangedGroups($templates, $db_templates, $del_objectids);
+		$this->addUnchangedMacros($templates, $db_templates, $del_objectids);
+		$this->addUnchangedTemplates($templates, $db_templates, $del_objectids);
+	}
+
 	protected function addRelatedObjects(array $options, array $result) {
 		$result = parent::addRelatedObjects($options, $result);
 
-		// adding template groups
-		$this->addRelatedGroups($options, $result, 'selectGroups');
-		$this->addRelatedGroups($options, $result, 'selectTemplateGroups');
+		$this->addRelatedTemplateGroups($options, $result);
 
 		$templateids = array_keys($result);
 
@@ -1368,28 +1243,18 @@ class CTemplate extends CHostGeneral {
 		return $result;
 	}
 
-	/**
-	 * Adds related template groups requested by "select*" options to the resulting object set.
-	 *
-	 * @param array  $options [IN] Original input options.
-	 * @param array  $result  [IN/OUT] Result output.
-	 * @param string $option  [IN] Possible values:
-	 *                               - "selectGroups" (deprecated);
-	 *                               - "selectHostGroups" (or any other value).
-	 */
-	private function addRelatedGroups(array $options, array &$result, string $option): void {
-		if ($options[$option] === null || $options[$option] === API_OUTPUT_COUNT) {
+	private function addRelatedTemplateGroups(array $options, array &$result): void {
+		if ($options['selectTemplateGroups'] === null || $options['selectTemplateGroups'] === API_OUTPUT_COUNT) {
 			return;
 		}
 
-		$relationMap = $this->createRelationMap($result, 'hostid', 'groupid', 'hosts_groups');
+		$relation_map = $this->createRelationMap($result, 'hostid', 'groupid', 'hosts_groups');
 		$groups = API::TemplateGroup()->get([
-			'output' => $options[$option],
-			'groupids' => $relationMap->getRelatedIds(),
+			'output' => $options['selectTemplateGroups'],
+			'groupids' => $relation_map->getRelatedIds(),
 			'preservekeys' => true
 		]);
 
-		$output_tag = $option === 'selectGroups' ? 'groups' : 'templategroups';
-		$result = $relationMap->mapMany($result, $groups, $output_tag);
+		$result = $relation_map->mapMany($result, $groups, 'templategroups');
 	}
 }
