@@ -1,6 +1,6 @@
 <?php
 /*
-** Copyright (C) 2001-2024 Zabbix SIA
+** Copyright (C) 2001-2025 Zabbix SIA
 **
 ** This program is free software: you can redistribute it and/or modify it under the terms of
 ** the GNU Affero General Public License as published by the Free Software Foundation, version 3.
@@ -79,7 +79,6 @@ class CTriggerPrototype extends CTriggerGeneral {
 			// output
 			'expandExpression'				=> null,
 			'output'						=> API_OUTPUT_EXTEND,
-			'selectGroups'					=> null,
 			'selectHostGroups'				=> null,
 			'selectTemplateGroups'			=> null,
 			'selectHosts'					=> null,
@@ -98,8 +97,6 @@ class CTriggerPrototype extends CTriggerGeneral {
 		];
 		$options = zbx_array_merge($defOptions, $options);
 
-		$this->checkDeprecatedParam($options, 'selectGroups');
-
 		// editable + permission check
 		if (self::$userData['type'] != USER_TYPE_SUPER_ADMIN && !$options['nopermissions']) {
 			if (self::$userData['ugsetid'] == 0) {
@@ -415,9 +412,8 @@ class CTriggerPrototype extends CTriggerGeneral {
 			}
 		}
 
-		// removing keys (hash -> array)
 		if (!$options['preservekeys']) {
-			$result = zbx_cleanHashes($result);
+			$result = array_values($result);
 		}
 
 		return $result;
@@ -521,7 +517,7 @@ class CTriggerPrototype extends CTriggerGeneral {
 	}
 
 	/**
-	 * Retrieves and adds additional requested data (options 'selectHosts', 'selectGroups', etc.) to result set.
+	 * Retrieves and adds additional requested data (options 'selectHosts', etc.) to result set.
 	 *
 	 * @param array		$options
 	 * @param array		$result
