@@ -1,21 +1,16 @@
 <?php
 /*
-** Zabbix
-** Copyright (C) 2001-2022 Zabbix SIA
+** Copyright (C) 2001-2025 Zabbix SIA
 **
-** This program is free software; you can redistribute it and/or modify
-** it under the terms of the GNU General Public License as published by
-** the Free Software Foundation; either version 2 of the License, or
-** (at your option) any later version.
+** This program is free software: you can redistribute it and/or modify it under the terms of
+** the GNU Affero General Public License as published by the Free Software Foundation, version 3.
 **
-** This program is distributed in the hope that it will be useful,
-** but WITHOUT ANY WARRANTY; without even the implied warranty of
-** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-** GNU General Public License for more details.
+** This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
+** without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+** See the GNU Affero General Public License for more details.
 **
-** You should have received a copy of the GNU General Public License
-** along with this program; if not, write to the Free Software
-** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+** You should have received a copy of the GNU Affero General Public License along with this program.
+** If not, see <https://www.gnu.org/licenses/>.
 **/
 
 
@@ -48,7 +43,8 @@ class CUserMacro extends CApiService {
 	 * @param array $options['globalmacroids'] global macros ids
 	 * @param array $options['templateids'] template ids
 	 * @param boolean $options['globalmacro'] only global macros
-	 * @param boolean $options['selectGroups'] select groups
+	 * @param boolean $options['selectHostGroups'] select host groups
+	 * @param boolean $options['selectTemplateGroups'] select template groups
 	 * @param boolean $options['selectHosts'] select hosts
 	 * @param boolean $options['selectTemplates'] select templates
 	 *
@@ -56,7 +52,6 @@ class CUserMacro extends CApiService {
 	 */
 	public function get($options = []) {
 		$result = [];
-		$userid = self::$userData['userid'];
 
 		$sqlParts = [
 			'select'	=> ['macros' => 'hm.hostmacroid'],
@@ -93,7 +88,8 @@ class CUserMacro extends CApiService {
 			'searchWildcardsEnabled'	=> null,
 			// output
 			'output'					=> API_OUTPUT_EXTEND,
-			'selectGroups'				=> null,
+			'selectHostGroups'			=> null,
+			'selectTemplateGroups'		=> null,
 			'selectHosts'				=> null,
 			'selectTemplates'			=> null,
 			'countOutput'				=> false,
@@ -106,25 +102,18 @@ class CUserMacro extends CApiService {
 
 		// editable + PERMISSION CHECK
 		if (self::$userData['type'] != USER_TYPE_SUPER_ADMIN && !$options['nopermissions']) {
-			if ($options['editable'] && !is_null($options['globalmacro'])) {
-				return [];
+			if (($options['editable'] && !is_null($options['globalmacro'])) || self::$userData['ugsetid'] == 0) {
+				return $options['countOutput'] ? '0' : [];
 			}
 			else {
-				$permission = $options['editable'] ? PERM_READ_WRITE : PERM_READ;
+				$sqlParts['from'][] = 'host_hgset hh';
+				$sqlParts['from'][] = 'permission p';
+				$sqlParts['where'][] = 'hm.hostid=hh.hostid';
+				$sqlParts['where'][] = 'hh.hgsetid=p.hgsetid';
 
-				$userGroups = getUserGroupsByUserId($userid);
-
-				$sqlParts['where'][] = 'EXISTS ('.
-						'SELECT NULL'.
-						' FROM hosts_groups hgg'.
-							' JOIN rights r'.
-								' ON r.id=hgg.groupid'.
-									' AND '.dbConditionInt('r.groupid', $userGroups).
-						' WHERE hm.hostid=hgg.hostid'.
-						' GROUP BY hgg.hostid'.
-						' HAVING MIN(r.permission)>'.PERM_DENY.
-							' AND MAX(r.permission)>='.zbx_dbstr($permission).
-						')';
+				if ($options['editable']) {
+					$sqlParts['where'][] = 'p.permission='.PERM_READ_WRITE;
+				}
 			}
 		}
 
@@ -135,7 +124,8 @@ class CUserMacro extends CApiService {
 			$options['triggerids'] = null;
 			$options['hostids'] = null;
 			$options['itemids'] = null;
-			$options['selectGroups'] = null;
+			$options['selectHostGroups'] = null;
+			$options['selectTemplateGroups'] = null;
 			$options['selectTemplates'] = null;
 			$options['selectHosts'] = null;
 			$options['inherited'] = null;
@@ -231,11 +221,10 @@ class CUserMacro extends CApiService {
 		if ($result) {
 			$result = $this->addRelatedObjects($options, $result);
 			$result = $this->unsetExtraFields($result, ['hostid', 'type'], $options['output']);
-		}
 
-		// removing keys (hash -> array)
-		if (!$options['preservekeys']) {
-			$result = zbx_cleanHashes($result);
+			if (!$options['preservekeys']) {
+				$result = array_values($result);
+			}
 		}
 
 		return $result;
@@ -249,14 +238,14 @@ class CUserMacro extends CApiService {
 	public function createGlobal(array $globalmacros) {
 		$this->validateCreateGlobal($globalmacros);
 
-		$globalmacroids = DB::insertBatch('globalmacro', $globalmacros);
+		$globalmacroids = DB::insert('globalmacro', $globalmacros);
 
 		foreach ($globalmacros as $index => &$globalmacro) {
 			$globalmacro['globalmacroid'] = $globalmacroids[$index];
 		}
 		unset($globalmacro);
 
-		$this->addAuditBulk(AUDIT_ACTION_ADD, AUDIT_RESOURCE_MACRO, $globalmacros);
+		self::addAuditLog(CAudit::ACTION_ADD, CAudit::RESOURCE_MACRO, $globalmacros);
 
 		return ['globalmacroids' => $globalmacroids];
 	}
@@ -267,16 +256,12 @@ class CUserMacro extends CApiService {
 	 * @throws APIException if the input is invalid.
 	 */
 	private function validateCreateGlobal(array &$globalmacros) {
-		if (self::$userData['type'] != USER_TYPE_SUPER_ADMIN) {
-			self::exception(ZBX_API_ERROR_PERMISSIONS, _('You do not have permission to perform this operation.'));
-		}
-
 		$api_input_rules = ['type' => API_OBJECTS, 'flags' => API_NOT_EMPTY | API_NORMALIZE, 'uniq' => [['macro']], 'fields' => [
 			'macro' =>			['type' => API_USER_MACRO, 'flags' => API_REQUIRED, 'length' => DB::getFieldLength('globalmacro', 'macro')],
 			'type' =>			['type' => API_INT32, 'in' => implode(',', [ZBX_MACRO_TYPE_TEXT, ZBX_MACRO_TYPE_SECRET, ZBX_MACRO_TYPE_VAULT]), 'default' => ZBX_MACRO_TYPE_TEXT],
 			'value' =>			['type' => API_MULTIPLE, 'flags' => API_REQUIRED, 'rules' => [
 									['if' => ['field' => 'type', 'in' => implode(',', [ZBX_MACRO_TYPE_TEXT, ZBX_MACRO_TYPE_SECRET])], 'type' => API_STRING_UTF8, 'length' => DB::getFieldLength('globalmacro', 'value')],
-									['if' => ['field' => 'type', 'in' => implode(',', [ZBX_MACRO_TYPE_VAULT])], 'type' => API_VAULT_SECRET, 'length' => DB::getFieldLength('globalmacro', 'value')]
+									['if' => ['field' => 'type', 'in' => ZBX_MACRO_TYPE_VAULT], 'type' => API_VAULT_SECRET, 'provider' => CSettingsHelper::get(CSettingsHelper::VAULT_PROVIDER), 'length' => DB::getFieldLength('globalmacro', 'value')]
 			]],
 			'description' =>	['type' => API_STRING_UTF8, 'length' => DB::getFieldLength('globalmacro', 'description')]
 		]];
@@ -315,7 +300,7 @@ class CUserMacro extends CApiService {
 			DB::update('globalmacro', $upd_globalmacros);
 		}
 
-		$this->addAuditBulk(AUDIT_ACTION_UPDATE, AUDIT_RESOURCE_MACRO, $globalmacros, $db_globalmacros);
+		self::addAuditLog(CAudit::ACTION_UPDATE, CAudit::RESOURCE_MACRO, $globalmacros, $db_globalmacros);
 
 		return ['globalmacroids' => array_column($globalmacros, 'globalmacroid')];
 	}
@@ -327,10 +312,6 @@ class CUserMacro extends CApiService {
 	 * @throws APIException if the input is invalid
 	 */
 	private function validateUpdateGlobal(array &$globalmacros, array &$db_globalmacros = null) {
-		if (self::$userData['type'] != USER_TYPE_SUPER_ADMIN) {
-			self::exception(ZBX_API_ERROR_PERMISSIONS, _('You do not have permission to perform this operation.'));
-		}
-
 		$api_input_rules = ['type' => API_OBJECTS, 'flags' => API_NOT_EMPTY | API_NORMALIZE, 'uniq' => [['globalmacroid'], ['macro']], 'fields' => [
 			'globalmacroid' =>	['type' => API_ID, 'flags' => API_REQUIRED],
 			'macro' =>			['type' => API_USER_MACRO, 'length' => DB::getFieldLength('globalmacro', 'macro')],
@@ -369,8 +350,10 @@ class CUserMacro extends CApiService {
 			}
 
 			if (array_key_exists('value', $globalmacro) && $globalmacro['type'] == ZBX_MACRO_TYPE_VAULT) {
-				if (!CApiInputValidator::validate(['type' => API_VAULT_SECRET], $globalmacro['value'],
-						'/'.($index + 1).'/value', $error)) {
+				if (!CApiInputValidator::validate([
+							'type' => API_VAULT_SECRET,
+							'provider' => CSettingsHelper::get(CSettingsHelper::VAULT_PROVIDER)
+						], $globalmacro['value'], '/'.($index + 1).'/value', $error)) {
 					self::exception(ZBX_API_ERROR_PARAMETERS, $error);
 				}
 			}
@@ -405,13 +388,12 @@ class CUserMacro extends CApiService {
 			return;
 		}
 
-		$db_globalmacros = DB::select('globalmacro', [
-			'output' => ['macro']
-		]);
-
 		$db_macros = [];
 
-		foreach ($db_globalmacros as $db_globalmacro) {
+		$options = ['output' => ['macro']];
+		$db_globalmacros = DBselect(DB::makeSql('globalmacro', $options));
+
+		while ($db_globalmacro = DBfetch($db_globalmacros)) {
 			$db_macros[CApiInputValidator::trimMacro($db_globalmacro['macro'])] = true;
 		}
 
@@ -432,7 +414,7 @@ class CUserMacro extends CApiService {
 
 		DB::delete('globalmacro', ['globalmacroid' => $globalmacroids]);
 
-		$this->addAuditBulk(AUDIT_ACTION_DELETE, AUDIT_RESOURCE_MACRO, $db_globalmacros);
+		self::addAuditLog(CAudit::ACTION_DELETE, CAudit::RESOURCE_MACRO, $db_globalmacros);
 
 		return ['globalmacroids' => $globalmacroids];
 	}
@@ -443,10 +425,6 @@ class CUserMacro extends CApiService {
 	 * @throws APIException if the input is invalid.
 	 */
 	private function validateDeleteGlobal(array &$globalmacroids, array &$db_globalmacros = null) {
-		if (self::$userData['type'] != USER_TYPE_SUPER_ADMIN) {
-			self::exception(ZBX_API_ERROR_PERMISSIONS, _('You do not have permission to perform this operation.'));
-		}
-
 		$api_input_rules = ['type' => API_IDS, 'flags' => API_NOT_EMPTY, 'uniq' => true];
 
 		if (!CApiInputValidator::validate($api_input_rules, $globalmacroids, '/', $error)) {
@@ -476,7 +454,7 @@ class CUserMacro extends CApiService {
 			'type' =>			['type' => API_INT32, 'in' => implode(',', [ZBX_MACRO_TYPE_TEXT, ZBX_MACRO_TYPE_SECRET, ZBX_MACRO_TYPE_VAULT]), 'default' => ZBX_MACRO_TYPE_TEXT],
 			'value' =>			['type' => API_MULTIPLE, 'flags' => API_REQUIRED, 'rules' => [
 									['if' => ['field' => 'type', 'in' => implode(',', [ZBX_MACRO_TYPE_TEXT, ZBX_MACRO_TYPE_SECRET])], 'type' => API_STRING_UTF8, 'length' => DB::getFieldLength('hostmacro', 'value')],
-									['if' => ['field' => 'type', 'in' => implode(',', [ZBX_MACRO_TYPE_VAULT])], 'type' => API_VAULT_SECRET, 'length' => DB::getFieldLength('hostmacro', 'value')]
+									['if' => ['field' => 'type', 'in' => implode(',', [ZBX_MACRO_TYPE_VAULT])], 'type' => API_VAULT_SECRET, 'provider' => CSettingsHelper::get(CSettingsHelper::VAULT_PROVIDER), 'length' => DB::getFieldLength('hostmacro', 'value')]
 			]],
 			'description' =>	['type' => API_STRING_UTF8, 'length' => DB::getFieldLength('hostmacro', 'description')]
 		]];
@@ -531,7 +509,8 @@ class CUserMacro extends CApiService {
 			'macro' =>			['type' => API_USER_MACRO, 'length' => DB::getFieldLength('hostmacro', 'macro')],
 			'type' =>			['type' => API_INT32, 'in' => implode(',', [ZBX_MACRO_TYPE_TEXT, ZBX_MACRO_TYPE_SECRET, ZBX_MACRO_TYPE_VAULT])],
 			'value' =>			['type' => API_STRING_UTF8, 'length' => DB::getFieldLength('hostmacro', 'value')],
-			'description' =>	['type' => API_STRING_UTF8, 'length' => DB::getFieldLength('hostmacro', 'description')]
+			'description' =>	['type' => API_STRING_UTF8, 'length' => DB::getFieldLength('hostmacro', 'description')],
+			'automatic' =>		['type' => API_INT32, 'in' => implode(',', [ZBX_USERMACRO_MANUAL])]
 		]];
 
 		if (!CApiInputValidator::validate($api_input_rules, $hostmacros, '/', $error)) {
@@ -539,7 +518,7 @@ class CUserMacro extends CApiService {
 		}
 
 		$db_hostmacros = $this->get([
-			'output' => ['hostmacroid', 'hostid', 'macro', 'type', 'description'],
+			'output' => ['hostmacroid', 'hostid', 'macro', 'type', 'description', 'automatic'],
 			'hostmacroids' => array_column($hostmacros, 'hostmacroid'),
 			'editable' => true,
 			'inherited' => false,
@@ -566,6 +545,12 @@ class CUserMacro extends CApiService {
 		foreach ($hostmacros as $index => &$hostmacro) {
 			$db_hostmacro = $db_hostmacros[$hostmacro['hostmacroid']];
 
+			if ($db_hostmacro['automatic'] == ZBX_USERMACRO_AUTOMATIC && !array_key_exists('automatic', $hostmacro)) {
+				self::exception(ZBX_API_ERROR_PERMISSIONS,
+					_s('Not allowed to modify automatic user macro "%1$s".', $db_hostmacro['macro'])
+				);
+			}
+
 			if ($hostmacro['type'] != $db_hostmacro['type']) {
 				if ($db_hostmacro['type'] == ZBX_MACRO_TYPE_SECRET) {
 					$hostmacro += ['value' => ''];
@@ -577,8 +562,10 @@ class CUserMacro extends CApiService {
 			}
 
 			if (array_key_exists('value', $hostmacro) && $hostmacro['type'] == ZBX_MACRO_TYPE_VAULT) {
-				if (!CApiInputValidator::validate(['type' => API_VAULT_SECRET], $hostmacro['value'],
-						'/'.($index + 1).'/value', $error)) {
+				if (!CApiInputValidator::validate([
+							'type' => API_VAULT_SECRET,
+							'provider' => CSettingsHelper::get(CSettingsHelper::VAULT_PROVIDER)
+						], $hostmacro['value'], '/'.($index + 1).'/value', $error)) {
 					self::exception(ZBX_API_ERROR_PARAMETERS, $error);
 				}
 			}
@@ -774,7 +761,7 @@ class CUserMacro extends CApiService {
 			'countOutput' => true,
 			'hostids' => $hostids,
 			'filter' => [
-				'flags' => ZBX_FLAG_DISCOVERY_NORMAL
+				'flags' => [ZBX_FLAG_DISCOVERY_NORMAL, ZBX_FLAG_DISCOVERY_CREATED]
 			],
 			'editable' => true
 		]);
@@ -992,7 +979,8 @@ class CUserMacro extends CApiService {
 		$sqlParts = parent::applyQueryOutputOptions($tableName, $tableAlias, $options, $sqlParts);
 
 		if ($options['output'] != API_OUTPUT_COUNT && $options['globalmacro'] === null) {
-			if ($options['selectGroups'] !== null || $options['selectHosts'] !== null || $options['selectTemplates'] !== null) {
+			if ($options['selectHostGroups'] !== null || $options['selectTemplateGroups'] !== null
+					|| $options['selectHosts'] !== null || $options['selectTemplates'] !== null) {
 				$sqlParts = $this->addQuerySelect($this->fieldId('hostid'), $sqlParts);
 			}
 		}
@@ -1045,59 +1033,108 @@ class CUserMacro extends CApiService {
 		$result = parent::addRelatedObjects($options, $result);
 
 		if ($options['globalmacro'] === null) {
-			$hostMacroIds = array_keys($result);
-
-			/*
-			 * Adding objects
-			 */
-			// adding groups
-			if ($options['selectGroups'] !== null && $options['selectGroups'] != API_OUTPUT_COUNT) {
-				$res = DBselect(
-					'SELECT hm.hostmacroid,hg.groupid'.
-						' FROM hostmacro hm,hosts_groups hg'.
-						' WHERE '.dbConditionInt('hm.hostmacroid', $hostMacroIds).
-						' AND hm.hostid=hg.hostid'
-				);
-				$relationMap = new CRelationMap();
-				while ($relation = DBfetch($res)) {
-					$relationMap->addRelation($relation['hostmacroid'], $relation['groupid']);
-				}
-
-				$groups = API::HostGroup()->get([
-					'output' => $options['selectGroups'],
-					'groupids' => $relationMap->getRelatedIds(),
-					'preservekeys' => true
-				]);
-				$result = $relationMap->mapMany($result, $groups, 'groups');
-			}
-
-			// adding templates
-			if ($options['selectTemplates'] !== null && $options['selectTemplates'] != API_OUTPUT_COUNT) {
-				$relationMap = $this->createRelationMap($result, 'hostmacroid', 'hostid');
-				$templates = API::Template()->get([
-					'output' => $options['selectTemplates'],
-					'templateids' => $relationMap->getRelatedIds(),
-					'preservekeys' => true
-				]);
-				$result = $relationMap->mapMany($result, $templates, 'templates');
-			}
-
-			// adding templates
-			if ($options['selectHosts'] !== null && $options['selectHosts'] != API_OUTPUT_COUNT) {
-				$relationMap = $this->createRelationMap($result, 'hostmacroid', 'hostid');
-				$templates = API::Host()->get([
-					'output' => $options['selectHosts'],
-					'hostids' => $relationMap->getRelatedIds(),
-					'preservekeys' => true
-				]);
-				$result = $relationMap->mapMany($result, $templates, 'hosts');
-			}
+			$this->addRelatedHostGroups($options, $result);
+			$this->addRelatedTemplateGroups($options, $result);
+			$this->addRelatedTemplates($options, $result);
+			$this->addRelatedHosts($options, $result);
 		}
 
 		return $result;
 	}
 
-	protected function unsetExtraFields(array $objects, array $fields, $output) {
+	private function addRelatedHostGroups(array $options, array &$result): void {
+		if ($options['selectHostGroups'] === null || $options['selectHostGroups'] === API_OUTPUT_COUNT) {
+			return;
+		}
+
+		$resource = DBselect(
+			'SELECT hm.hostmacroid,hg.groupid'.
+			' FROM hostmacro hm'.
+			' JOIN hosts_groups hg ON hm.hostid=hg.hostid'.
+			' JOIN hstgrp hgg ON hg.groupid=hgg.groupid'.
+				' AND '.dbConditionInt('hgg.type', [HOST_GROUP_TYPE_HOST_GROUP]).
+			' WHERE '.dbConditionId('hm.hostmacroid', array_keys($result))
+		);
+		$relation_map = new CRelationMap();
+
+		while ($relation = DBfetch($resource)) {
+			$relation_map->addRelation($relation['hostmacroid'], $relation['groupid']);
+		}
+
+		$related_ids = $relation_map->getRelatedIds();
+		$groups = $related_ids
+			? API::HostGroup()->get([
+				'output' => $options['selectHostGroups'],
+				'groupids' => $related_ids,
+				'preservekeys' => true
+			])
+			: [];
+
+		$result = $relation_map->mapMany($result, $groups, 'hostgroups');
+	}
+
+	private function addRelatedTemplateGroups(array $options, array &$result): void {
+		if ($options['selectTemplateGroups'] === null || $options['selectTemplateGroups'] === API_OUTPUT_COUNT) {
+			return;
+		}
+
+		$resource = DBselect(
+			'SELECT hm.hostmacroid,hg.groupid'.
+			' FROM hostmacro hm'.
+			' JOIN hosts_groups hg ON hm.hostid=hg.hostid'.
+			' JOIN hstgrp hgg ON hg.groupid=hgg.groupid'.
+				' AND '.dbConditionInt('hgg.type', [HOST_GROUP_TYPE_TEMPLATE_GROUP]).
+			' WHERE '.dbConditionId('hm.hostmacroid', array_keys($result))
+		);
+		$relation_map = new CRelationMap();
+
+		while ($relation = DBfetch($resource)) {
+			$relation_map->addRelation($relation['hostmacroid'], $relation['groupid']);
+		}
+
+		$related_ids = $relation_map->getRelatedIds();
+		$groups = $related_ids
+			? API::TemplateGroup()->get([
+				'output' => $options['selectTemplateGroups'],
+				'groupids' => $related_ids,
+				'preservekeys' => true
+			])
+			: [];
+
+		$result = $relation_map->mapMany($result, $groups, 'templategroups');
+	}
+
+	private function addRelatedTemplates(array $options, array &$result): void {
+		if ($options['selectTemplates'] === null || $options['selectTemplates'] === API_OUTPUT_COUNT) {
+			return;
+		}
+
+		$relation_map = $this->createRelationMap($result, 'hostmacroid', 'hostid');
+		$templates = API::Template()->get([
+			'output' => $options['selectTemplates'],
+			'templateids' => $relation_map->getRelatedIds(),
+			'preservekeys' => true
+		]);
+
+		$result = $relation_map->mapMany($result, $templates, 'templates');
+	}
+
+	private function addRelatedHosts(array $options, array &$result): void {
+		if ($options['selectHosts'] === null || $options['selectHosts'] === API_OUTPUT_COUNT) {
+			return;
+		}
+
+		$relation_map = $this->createRelationMap($result, 'hostmacroid', 'hostid');
+		$hosts = API::Host()->get([
+			'output' => $options['selectHosts'],
+			'hostids' => $relation_map->getRelatedIds(),
+			'preservekeys' => true
+		]);
+
+		$result = $relation_map->mapMany($result, $hosts, 'hosts');
+	}
+
+	protected function unsetExtraFields(array $objects, array $fields, $output = []) {
 		foreach ($objects as &$object) {
 			if ($object['type'] == ZBX_MACRO_TYPE_SECRET) {
 				unset($object['value']);
