@@ -444,10 +444,7 @@ function make_small_eventlist(array $startEvent, array $allowed) {
 		// Create acknowledge link.
 		$problem_update_link = ($allowed['add_comments'] || $allowed['change_severity'] || $allowed['acknowledge']
 				|| $can_be_closed || $allowed['suppress_problems'] || $allowed['rank_change'])
-			? (new CLink(_('Update'), $problem_update_url))
-				->addClass(ZBX_STYLE_LINK_ALT)
-				->setAttribute('data-eventids[]', $event['eventid'])
-				->setAttribute('data-action', 'acknowledge.edit')
+			? (new CLink(_('Update'), $problem_update_url))->addClass(ZBX_STYLE_LINK_ALT)
 			: new CSpan(_('Update'));
 
 		$table->addRow([
