@@ -1,6 +1,6 @@
 <?php
 /*
-** Copyright (C) 2001-2024 Zabbix SIA
+** Copyright (C) 2001-2025 Zabbix SIA
 **
 ** This program is free software: you can redistribute it and/or modify it under the terms of
 ** the GNU Affero General Public License as published by the Free Software Foundation, version 3.
@@ -1010,10 +1010,7 @@ function makeParentTemplatesList(array $parent_templates): array {
 				->setArgument('templateid', $templateid)
 				->getUrl();
 
-			$template_list[] = (new CLink($template['name'], $template_url))
-				->setAttribute('data-templateid', $templateid)
-				->setAttribute('data-action', 'template.edit')
-				->addClass('js-edit-template');
+			$template_list[] = new CLink($template['name'], $template_url);
 		}
 		else {
 			$template_list[] = (new CSpan($template['name']))->addClass(ZBX_STYLE_GREY);
