@@ -1,6 +1,6 @@
 <?php
 /*
-** Copyright (C) 2001-2024 Zabbix SIA
+** Copyright (C) 2001-2025 Zabbix SIA
 **
 ** This program is free software: you can redistribute it and/or modify it under the terms of
 ** the GNU Affero General Public License as published by the Free Software Foundation, version 3.
@@ -292,11 +292,13 @@ function getHostNavigation(string $current_element, $hostid, $lld_ruleid = 0): ?
 	$list = new CList();
 
 	if ($is_template) {
-		$template = new CSpan(
-			(new CLink($db_host['name']))
-				->setAttribute('data-templateid', $db_host['templateid'])
-				->onClick('view.editTemplate(event, this.dataset.templateid);')
-		);
+		$template_url = (new CUrl('zabbix.php'))
+			->setArgument('action', 'popup')
+			->setArgument('popup', 'template.edit')
+			->setArgument('templateid', $db_host['templateid'])
+			->getUrl();
+
+		$template = new CSpan((new CLink($db_host['name'], $template_url)));
 
 		if ($current_element === '') {
 			$template->addClass(ZBX_STYLE_SELECTED);
@@ -327,15 +329,13 @@ function getHostNavigation(string $current_element, $hostid, $lld_ruleid = 0): ?
 				break;
 		}
 
-		$host = new CSpan(
-			(new CLink($db_host['name'],
-				(new CUrl('zabbix.php'))
-					->setArgument('action', 'host.edit')
-					->setArgument('hostid', $db_host['hostid'])
-			))
-				->setAttribute('data-hostid', $db_host['hostid'])
-				->onClick('view.editHost(event, this.dataset.hostid);')
-		);
+		$host_url = (new CUrl('zabbix.php'))
+			->setArgument('action', 'popup')
+			->setArgument('popup', 'host.edit')
+			->setArgument('hostid', $db_host['hostid'])
+			->getUrl();
+
+		$host = new CSpan((new CLink($db_host['name'], $host_url)));
 
 		if ($current_element === '') {
 			$host->addClass(ZBX_STYLE_SELECTED);
@@ -824,7 +824,7 @@ function makeLogo(int $type): CTag {
 	$brand_logo = CBrandHelper::getLogo($type);
 
 	if ($brand_logo !== null) {
-		return (new CImg($brand_logo));
+		return (new CImg($brand_logo))->addClass($zabbix_logo_classes[$type]);
 	}
 
 	return (new CDiv())->addClass($zabbix_logo_classes[$type]);
